"""
Tests basic ThreadSanitizer support (detecting a data race).
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import json


class TsanBasicTestCase(TestBase):
    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="non-core functionality, need to reenable and fix later (DES 2014.11.07)",
    )
    @expectedFailureNetBSD
    @skipIfFreeBSD  # llvm.org/pr21136 runtimes not yet available by default
    @skipIfRemote
    @skipUnlessThreadSanitizer
    @no_debug_info_test
    def test(self):
        self.build()
        self.tsan_tests()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.line_malloc = line_number("main.c", "// malloc line")
        self.line_thread1 = line_number("main.c", "// thread1 line")
        self.line_thread2 = line_number("main.c", "// thread2 line")

    def tsan_tests(self):
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.runCmd("run")

        stop_reason = (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        )
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        # the stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            "A data race should be detected",
            substrs=["stopped", "stop reason = Data race detected"],
        )

        self.assertEqual(
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason(),
            lldb.eStopReasonInstrumentation,
        )

        # test that the TSan dylib is present
        self.expect(
            "image lookup -n __tsan_get_current_report",
            "__tsan_get_current_report should be present",
            substrs=["1 match found"],
        )

        process = self.dbg.GetSelectedTarget().process
        thread = process.GetSelectedThread()
        frame = thread.GetSelectedFrame()
        if self.platformIsDarwin():
            # We should not be stopped in the sanitizer library.
            self.assertIn("f2", frame.GetFunctionName())
        else:
            self.assertIn("__tsan_on_report", frame.GetFunctionName())

        # The stopped thread backtrace should contain either line1 or line2
        # from main.c.
        found = False
        for i in range(0, thread.GetNumFrames()):
            frame = thread.GetFrameAtIndex(i)
            if frame.GetLineEntry().GetFileSpec().GetFilename() == "main.c":
                if frame.GetLineEntry().GetLine() == self.line_thread1:
                    found = True
                if frame.GetLineEntry().GetLine() == self.line_thread2:
                    found = True
        self.assertTrue(found)

        self.expect(
            "thread info -s",
            "The extended stop info should contain the TSan provided fields",
            substrs=["instrumentation_class", "description", "mops"],
        )

        output_lines = self.res.GetOutput().split("\n")
        json_line = "\n".join(output_lines[2:])
        data = json.loads(json_line)
        self.assertEqual(data["instrumentation_class"], "ThreadSanitizer")
        self.assertEqual(data["issue_type"], "data-race")
        self.assertEqual(len(data["mops"]), 2)

        backtraces = thread.GetStopReasonExtendedBacktraces(
            lldb.eInstrumentationRuntimeTypeAddressSanitizer
        )
        self.assertEqual(backtraces.GetSize(), 0)

        backtraces = thread.GetStopReasonExtendedBacktraces(
            lldb.eInstrumentationRuntimeTypeThreadSanitizer
        )
        self.assertGreaterEqual(backtraces.GetSize(), 2)

        # First backtrace is a memory operation
        thread = backtraces.GetThreadAtIndex(0)
        found = False
        for i in range(0, thread.GetNumFrames()):
            frame = thread.GetFrameAtIndex(i)
            if frame.GetLineEntry().GetFileSpec().GetFilename() == "main.c":
                if frame.GetLineEntry().GetLine() == self.line_thread1:
                    found = True
                if frame.GetLineEntry().GetLine() == self.line_thread2:
                    found = True
        self.assertTrue(found)

        # Second backtrace is a memory operation
        thread = backtraces.GetThreadAtIndex(1)
        found = False
        for i in range(0, thread.GetNumFrames()):
            frame = thread.GetFrameAtIndex(i)
            if frame.GetLineEntry().GetFileSpec().GetFilename() == "main.c":
                if frame.GetLineEntry().GetLine() == self.line_thread1:
                    found = True
                if frame.GetLineEntry().GetLine() == self.line_thread2:
                    found = True
        self.assertTrue(found)

        self.runCmd("continue")

        # the stop reason of the thread should be a SIGABRT.
        self.expect(
            "thread list",
            "We should be stopped due a SIGABRT",
            substrs=["stopped", "stop reason = signal SIGABRT"],
        )

        # test that we're in pthread_kill now (TSan abort the process)
        self.expect(
            "thread list",
            "We should be stopped in pthread_kill",
            substrs=["pthread_kill"],
        )



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.
from array import array

from mlir.ir import *
from mlir.dialects import rocdl, arith
from mlir.extras import types as T


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: testSmoke
@constructAndPrintInModule
def testSmoke():
    v_len = 16
    f32 = F32Type.get()
    # Note: this isn't actually the right type for the intrinsic (should be f16)
    # but array doesn't support f16.
    v16f32 = T.vector(v_len, f32)
    f32_array = array("f", [0.0] * v_len)
    a_frag = arith.constant(v16f32, f32_array)
    b_frag = arith.constant(v16f32, f32_array)
    c_frag = arith.constant(v16f32, f32_array)

    c_frag = rocdl.wmma_f16_16x16x16_f16(v16f32, a_frag, b_frag, c_frag, opsel=False)
    # CHECK: %{{.*}} = "rocdl.wmma.f16.16x16x16.f16"
    print(c_frag)
    assert isinstance(c_frag, OpView)
    # CHECK: OpResult(%{{.*}} = "rocdl.wmma.f16.16x16x16.f16"
    c_frag = rocdl.wmma_f16_16x16x16_f16_(v16f32, a_frag, b_frag, c_frag, opsel=False)
    print(c_frag)
    assert isinstance(c_frag, Value)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class AproposTestCase(TestBase):
    @no_debug_info_test
    def test_apropos(self):
        self.expect(
            "apropos", error=True, substrs=[" must be called with exactly one argument"]
        )
        self.expect(
            "apropos a b",
            error=True,
            substrs=[" must be called with exactly one argument"],
        )
        self.expect("apropos ''", error=True, substrs=["'' is not a valid search word"])

    @no_debug_info_test
    def test_apropos_variable(self):
        """Test that 'apropos variable' prints the fully qualified command name"""
        self.expect(
            "apropos variable",
            substrs=["frame variable", "target variable", "watchpoint set variable"],
        )



# Module level initialization for the `lldbsuite` module.

import inspect
import os
import sys


def find_lldb_root():
    lldb_root = os.path.realpath(
        os.path.dirname(inspect.getfile(inspect.currentframe()))
    )
    while True:
        parent = os.path.dirname(lldb_root)
        if parent == lldb_root:  # dirname('/') == '/'
            raise Exception("use_lldb_suite_root.py not found")
        lldb_root = parent

        test_path = os.path.join(lldb_root, "use_lldb_suite_root.py")
        if os.path.isfile(test_path):
            return lldb_root


# lldbsuite.lldb_root refers to the root of the git/svn source checkout
lldb_root = find_lldb_root()

# lldbsuite.lldb_test_src_root refers to the root of the python test case tree
# (i.e. the actual unit tests).
lldb_test_root = os.path.join(lldb_root, "test", "API")



"""Test that C++ global variables can be inspected by name and also their mangled name."""


from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class GlobalVariablesCppTestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.source = lldb.SBFileSpec("main.cpp")

    def test(self):
        self.build()

        (target, _, _, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", self.source
        )

        # Check that we can access g_file_global_int by its name
        self.expect(
            "target variable g_file_global_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["42"],
        )
        self.expect(
            "target variable abc::g_file_global_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["42"],
        )
        self.expect(
            "target variable xyz::g_file_global_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            error=True,
            substrs=["can't find global variable"],
        )

        # Check that we can access g_file_global_const_int by its name
        self.expect(
            "target variable g_file_global_const_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1337"],
        )
        self.expect(
            "target variable abc::g_file_global_const_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["1337"],
        )
        self.expect(
            "target variable xyz::g_file_global_const_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            error=True,
            substrs=["can't find global variable"],
        )

        # Try accessing a global variable in anonymous namespace.
        self.expect(
            "target variable g_anon_namespace_const_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["100"],
        )
        self.expect(
            "target variable abc::g_anon_namespace_const_int",
            VARIABLES_DISPLAYED_CORRECTLY,
            error=True,
            substrs=["can't find global variable"],
        )
        var = target.FindFirstGlobalVariable(
            "abc::(anonymous namespace)::g_anon_namespace_const_int"
        )
        self.assertTrue(var.IsValid())
        self.assertEqual(
            var.GetName(), "abc::(anonymous namespace)::g_anon_namespace_const_int"
        )
        self.assertEqual(var.GetValue(), "100")

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24764")
    def test_access_by_mangled_name(self):
        self.build()

        (target, _, _, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", self.source
        )

        # Check that we can access g_file_global_int by its mangled name
        addr = target.EvaluateExpression("&abc::g_file_global_int").GetValueAsUnsigned()
        self.assertNotEqual(addr, 0)
        mangled = lldb.SBAddress(addr, target).GetSymbol().GetMangledName()
        self.assertNotEqual(mangled, None)
        gv = target.FindFirstGlobalVariable(mangled)
        self.assertTrue(gv.IsValid())
        self.assertEqual(gv.GetName(), "abc::g_file_global_int")
        self.assertEqual(gv.GetValueAsUnsigned(), 42)



"""
Tests that auto types work
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPAutoTestCase(TestBase):
    def test_with_run_command(self):
        """Test that auto types work in the expression parser"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("auto f = 123456; f", result_type="int", result_value="123456")
        self.expect(
            "expr struct Test { int x; int y; Test() : x(123), y(456) {} }; auto t = Test(); t",
            substrs=["Test", "123", "456"],
        )
        self.expect_expr("auto s = foo; s", result_type="long", result_value="1234")



"""
Test Process::FindInMemory.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from address_ranges_helper import *


class FindInMemoryTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)

        self.build()
        (
            self.target,
            self.process,
            self.thread,
            self.bp,
        ) = lldbutil.run_to_source_breakpoint(
            self,
            "break here",
            lldb.SBFileSpec("main.cpp"),
        )
        self.assertTrue(self.bp.IsValid())

    def test_check_stack_pointer(self):
        """Make sure the 'stack_pointer' variable lives on the stack"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        frame = self.thread.GetSelectedFrame()
        ex = frame.EvaluateExpression("&stack_pointer")
        variable_region = lldb.SBMemoryRegionInfo()
        self.assertTrue(
            self.process.GetMemoryRegionInfo(
                ex.GetValueAsUnsigned(), variable_region
            ).Success(),
        )

        stack_region = lldb.SBMemoryRegionInfo()
        self.assertTrue(
            self.process.GetMemoryRegionInfo(frame.GetSP(), stack_region).Success(),
        )

        self.assertEqual(variable_region, stack_region)

    def test_find_in_memory_ok(self):
        """Make sure a match exists in the heap memory and the right address ranges are provided"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        error = lldb.SBError()
        addr = self.process.FindInMemory(
            SINGLE_INSTANCE_PATTERN_STACK,
            GetStackRange(self, True),
            1,
            error,
        )

        self.assertSuccess(error)
        self.assertNotEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_find_in_memory_double_instance_ok(self):
        """Make sure a match exists in the heap memory and the right address ranges are provided"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        error = lldb.SBError()
        addr = self.process.FindInMemory(
            DOUBLE_INSTANCE_PATTERN_HEAP,
            GetHeapRanges(self, True)[0],
            1,
            error,
        )

        self.assertSuccess(error)
        self.assertNotEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_find_in_memory_invalid_alignment(self):
        """Make sure the alignment 0 is failing"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        error = lldb.SBError()
        addr = self.process.FindInMemory(
            SINGLE_INSTANCE_PATTERN_STACK,
            GetStackRange(self, True),
            0,
            error,
        )

        self.assertFailure(error)
        self.assertEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_find_in_memory_invalid_address_range(self):
        """Make sure invalid address range is failing"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        error = lldb.SBError()
        addr = self.process.FindInMemory(
            SINGLE_INSTANCE_PATTERN_STACK,
            lldb.SBAddressRange(),
            1,
            error,
        )

        self.assertFailure(error)
        self.assertEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_find_in_memory_invalid_buffer(self):
        """Make sure the empty buffer is failing"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        error = lldb.SBError()
        addr = self.process.FindInMemory(
            "",
            GetStackRange(self, True),
            1,
            error,
        )

        self.assertFailure(error)
        self.assertEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_find_in_memory_unaligned(self):
        """Make sure the unaligned match exists in the heap memory and is not found with alignment 8"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        error = lldb.SBError()
        range = GetAlignedRange(self, True)

        # First we make sure the pattern is found with alignment 1
        addr = self.process.FindInMemory(
            UNALIGNED_INSTANCE_PATTERN_HEAP,
            range,
            1,
            error,
        )
        self.assertSuccess(error)
        self.assertNotEqual(addr, lldb.LLDB_INVALID_ADDRESS)

        # With alignment 8 the pattern should not be found
        addr = self.process.FindInMemory(
            UNALIGNED_INSTANCE_PATTERN_HEAP,
            range,
            8,
            error,
        )
        self.assertSuccess(error)
        self.assertEqual(addr, lldb.LLDB_INVALID_ADDRESS)

    def test_memory_info_list_iterable(self):
        """Make sure the SBMemoryRegionInfoList is iterable and each yielded object is unique"""
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertState(self.process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        info_list = self.process.GetMemoryRegions()
        self.assertTrue(info_list.GetSize() > 0)

        collected_info = []
        try:
            for info in info_list:
                collected_info.append(info)
        except Exception:
            self.fail("SBMemoryRegionInfoList is not iterable")

        for i in range(len(collected_info)):
            region = lldb.SBMemoryRegionInfo()
            info_list.GetMemoryRegionAtIndex(i, region)

            self.assertEqual(
                collected_info[i],
                region,
                f"items {i}: iterator data should match index access data",
            )

        self.assertTrue(
            len(collected_info) >= 2, "Test requires at least 2 memory regions"
        )
        self.assertNotEqual(
            collected_info[0].GetRegionBase(),
            collected_info[1].GetRegionBase(),
            "Different items should have different base addresses",
        )

        self.assertEqual(
            info_list[0].GetRegionBase(),
            collected_info[0].GetRegionBase(),
            "subscript [0] should match first collected item",
        )
        self.assertEqual(
            info_list[-1].GetRegionBase(),
            collected_info[-1].GetRegionBase(),
            "subscript [-1] should match last collected item",
        )
        with self.assertRaises(IndexError):
            info_list[info_list.GetSize()]
        with self.assertRaises(TypeError):
            info_list["invalid"]



# FIXME: remove when LLDB_MINIMUM_PYTHON_VERSION > 3.8
from __future__ import annotations

import os
import time
from typing import Optional, Callable, Any, List, Mapping, Final
import uuid

import dap_server
from dap_server import Source, Response
from lldbsuite.test.decorators import skipIf
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbplatformutil
import lldbgdbserverutils
import base64


# DAP tests as a whole have been flakey on the Windows on Arm bot. See:
# https://github.com/llvm/llvm-project/issues/137660
@skipIf(oslist=["windows"], archs=["aarch64"])
class DAPTestCaseBase(TestBase):
    # set timeout based on whether ASAN was enabled or not. Increase
    # timeout by a factor of 10 if ASAN is enabled.
    DEFAULT_TIMEOUT: Final[float] = dap_server.DEFAULT_TIMEOUT
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        self.dap_server_count = 0
        super().setUp()

    def create_debug_adapter(
        self,
        lldbDAPEnv: Optional[dict[str, str]] = None,
        connection: Optional[str] = None,
        connection_timeout: Optional[int] = None,
        additional_args: Optional[list[str]] = None,
    ):
        """Create the Visual Studio Code debug adapter"""
        self.assertTrue(
            is_exe(self.lldbDAPExec), "lldb-dap must exist and be executable"
        )
        if self.dap_server_count:
            log_file_path = (
                self.getLogBasenameForCurrentTest()
                + f"-dap-{self.dap_server_count}.log"
            )
        else:
            log_file_path = self.getLogBasenameForCurrentTest() + "-dap.log"
        self.dap_server_count += 1
        self.dap_server = dap_server.DebugAdapterServer(
            executable=self.lldbDAPExec,
            connection=connection,
            connection_timeout=connection_timeout,
            init_commands=self.setUpCommands(),
            log_file=log_file_path,
            env=lldbDAPEnv,
            additional_args=additional_args,
            spawn_helper=self.spawnSubprocess,
        )
        self.log_files.append(log_file_path)

    def build_and_create_debug_adapter(
        self,
        lldbDAPEnv: Optional[dict[str, str]] = None,
        dictionary: Optional[dict] = None,
        additional_args: Optional[list[str]] = None,
    ):
        self.build(dictionary=dictionary)
        self.create_debug_adapter(lldbDAPEnv, additional_args=additional_args)

    def build_and_create_debug_adapter_for_attach(self):
        """Variant of build_and_create_debug_adapter that builds a uniquely
        named binary."""
        unique_name = str(uuid.uuid4())
        self.build_and_create_debug_adapter(dictionary={"EXE": unique_name})
        return self.getBuildArtifact(unique_name)

    def set_source_breakpoints(
        self, source_path, lines, data=None, wait_for_resolve=True
    ):
        """Sets source breakpoints and returns an array of strings containing
        the breakpoint IDs ("1", "2") for each breakpoint that was set.
        Parameter data is array of data objects for breakpoints.
        Each object in data is 1:1 mapping with the entry in lines.
        It contains optional location/hitCondition/logMessage parameters.
        """
        return self.set_source_breakpoints_from_source(
            Source(path=source_path), lines, data, wait_for_resolve
        )

    def set_source_breakpoints_assembly(
        self, source_reference, lines, data=None, wait_for_resolve=True
    ):
        return self.set_source_breakpoints_from_source(
            Source.build(source_reference=source_reference),
            lines,
            data,
            wait_for_resolve,
        )

    def set_source_breakpoints_from_source(
        self, source: Source, lines, data=None, wait_for_resolve=True
    ):
        response = self.dap_server.request_setBreakpoints(
            source,
            lines,
            data,
        )
        if response is None:
            return []
        breakpoints = response["body"]["breakpoints"]
        breakpoint_ids = [b["id"] for b in breakpoints]
        if wait_for_resolve:
            self.wait_for_breakpoints_to_resolve(breakpoint_ids)
        return breakpoint_ids

    def set_function_breakpoints(
        self, functions, condition=None, hitCondition=None, wait_for_resolve=True
    ):
        """Sets breakpoints by function name given an array of function names
        and returns an array of strings containing the breakpoint IDs
        ("1", "2") for each breakpoint that was set.
        """
        response = self.dap_server.request_setFunctionBreakpoints(
            functions, condition=condition, hitCondition=hitCondition
        )
        if response is None:
            return []
        breakpoints = response["body"]["breakpoints"]
        breakpoint_ids = [b["id"] for b in breakpoints]
        if wait_for_resolve:
            self.wait_for_breakpoints_to_resolve(breakpoint_ids)
        return breakpoint_ids

    def wait_for_breakpoints_to_resolve(self, breakpoint_ids: List[int]):
        unresolved_breakpoints = self.dap_server.wait_for_breakpoints_to_be_verified(
            breakpoint_ids
        )
        self.assertEqual(
            len(unresolved_breakpoints),
            0,
            f"Expected to resolve all breakpoints. Unresolved breakpoint ids: {unresolved_breakpoints}",
        )

    def wait_until(
        self,
        predicate: Callable[[], bool],
        delay: float = 0.5,
    ) -> bool:
        """Repeatedly run the predicate until either the predicate returns True
        or a timeout has occurred."""
        deadline = time.monotonic() + self.DEFAULT_TIMEOUT
        while deadline > time.monotonic():
            if predicate():
                return True
            time.sleep(delay)
        return False

    def assertCapabilityIsSet(self, key: str, msg: Optional[str] = None) -> None:
        """Assert that given capability is set in the client."""
        self.assertIn(key, self.dap_server.capabilities, msg)
        self.assertEqual(self.dap_server.capabilities[key], True, msg)

    def assertCapabilityIsNotSet(self, key: str, msg: Optional[str] = None) -> None:
        """Assert that given capability is not set in the client."""
        if key in self.dap_server.capabilities:
            self.assertEqual(self.dap_server.capabilities[key], False, msg)

    def verify_breakpoint_hit(self, breakpoint_ids: List[int]):
        """Wait for the process we are debugging to stop, and verify we hit
        any breakpoint location in the "breakpoint_ids" array.
        "breakpoint_ids" should be a list of breakpoint IDs
        ([1, 2]). The return value from self.set_source_breakpoints()
        or self.set_function_breakpoints() can be passed to this function"""
        if not all(isinstance(i, int) for i in breakpoint_ids):
            raise TypeError(f"breakpoint ids must be an int in {breakpoint_ids!r}")
        stopped_events = self.dap_server.wait_for_stopped()
        for stopped_event in stopped_events:
            if "body" in stopped_event:
                body = stopped_event["body"]
                if "reason" not in body:
                    continue
                if (
                    body["reason"] != "breakpoint"
                    and body["reason"] != "instruction breakpoint"
                    and body["reason"] != "data breakpoint"
                ):
                    continue
                if "hitBreakpointIds" not in body:
                    continue
                hit_breakpoint_ids = body["hitBreakpointIds"]
                for bp in hit_breakpoint_ids:
                    if bp in breakpoint_ids:
                        return
        self.fail(
            f"breakpoint not hit, wanted breakpoint_ids {breakpoint_ids} in stopped_events {stopped_events}",
        )

    def verify_all_breakpoints_hit(self, breakpoint_ids: List[int]) -> None:
        """Wait for the process we are debugging to stop, and verify we hit
        all of the breakpoint locations in the "breakpoint_ids" array.
        "breakpoint_ids" should be a list of int breakpoint IDs ([1, 2])."""
        if not all(isinstance(i, int) for i in breakpoint_ids):
            raise TypeError(f"breakpoint ids must be an int in {breakpoint_ids!r}")
        stopped_events = self.dap_server.wait_for_stopped()
        for stopped_event in stopped_events:
            if "body" in stopped_event:
                body = stopped_event["body"]
                if "reason" not in body:
                    continue
                if (
                    body["reason"] != "breakpoint"
                    and body["reason"] != "instruction breakpoint"
                ):
                    continue
                if "hitBreakpointIds" not in body:
                    continue
                hit_bps = body["hitBreakpointIds"]
                if all(breakpoint_id in hit_bps for breakpoint_id in breakpoint_ids):
                    return
        self.fail(f"breakpoints not hit, stopped_events={stopped_events}")

    def verify_stop_exception_info(
        self, expected_description: str, expected_text: Optional[str] = None
    ) -> None:
        """Wait for the debuggee to stop, and verify the stop reason is
        'exception' with the description matching 'expected_description' and
        text match 'expected_text', if specified."""
        stopped_events = self.dap_server.wait_for_stopped()
        self.assertIsNotNone(stopped_events, "No stopped events detected")
        for stopped_event in stopped_events:
            body = stopped_event["body"]
            if body["reason"] != "exception":
                continue
            self.assertIn(
                "description",
                body,
                f"stopped event missing description {stopped_event}",
            )
            description = body["description"]
            self.assertRegex(
                description,
                expected_description,
                f"for 'stopped' event {stopped_event!r}",
            )
            if expected_text:
                self.assertRegex(
                    body["text"],
                    expected_text,
                    f"for stopped event {stopped_event!r}",
                )
            return
        self.fail(f"No valid stop exception info detected in {stopped_events!r}")

    def verify_stop_on_entry(self) -> None:
        """Waits for the process to be stopped and then verifies at least one
        thread has the stop reason 'entry'."""
        if not self.dap_server.configuration_done_sent:
            self.verify_configuration_done()
        self.dap_server.wait_for_stopped()
        self.assertIn(
            "entry",
            (t["reason"] for t in self.dap_server.thread_stop_reasons.values()),
            f"Expected at least one thread to report stop reason 'entry' in {self.dap_server.thread_stop_reasons}",
        )

    def verify_commands(self, flavor: str, output: str, commands: List[str]):
        self.assertTrue(output and len(output) > 0, "expect console output")
        lines = output.splitlines()
        prefix = "(lldb) "
        for cmd in commands:
            found = False
            for line in lines:
                if len(cmd) > 0 and (cmd[0] == "!" or cmd[0] == "?"):
                    cmd = cmd[1:]
                if line.startswith(prefix) and cmd in line:
                    found = True
                    break
            self.assertTrue(
                found,
                f"Command '{flavor}' - '{cmd}' not found in output: {output}",
            )

    def get_dict_value(self, d: Mapping[str, Any], key_path: List[str]) -> Any:
        """Verify each key in the key_path array is in contained in each
        dictionary within "d". Assert if any key isn't in the
        corresponding dictionary. This is handy for grabbing values from VS
        Code response dictionary like getting
        response['body']['stackFrames']
        """
        value = d
        for key in key_path:
            self.assertIn(
                key, value, f"lookup failure for dict={d} key_path={key_path}"
            )
            value = value[key]
        return value

    def get_stackFrames_and_totalFramesCount(
        self, threadId=None, startFrame=None, levels=None, format=None, dump=False
    ):
        response = self.dap_server.request_stackTrace(
            threadId=threadId,
            startFrame=startFrame,
            levels=levels,
            format=format,
            dump=dump,
        )
        if response:
            stackFrames = self.get_dict_value(response, ["body", "stackFrames"])
            totalFrames = self.get_dict_value(response, ["body", "totalFrames"])
            self.assertTrue(
                totalFrames > 0,
                "verify totalFrames count is provided by extension that supports "
                "async frames loading",
            )
            return (stackFrames, totalFrames)
        return (None, 0)

    def get_stackFrames(
        self, threadId=None, startFrame=None, levels=None, format=None, dump=False
    ):
        (stackFrames, totalFrames) = self.get_stackFrames_and_totalFramesCount(
            threadId=threadId,
            startFrame=startFrame,
            levels=levels,
            format=format,
            dump=dump,
        )
        return stackFrames

    def get_exceptionInfo(self, threadId=None):
        response = self.dap_server.request_exceptionInfo(threadId=threadId)
        return self.get_dict_value(response, ["body"])

    def get_source_and_line(self, threadId=None, frameIndex=0):
        stackFrames = self.get_stackFrames(
            threadId=threadId, startFrame=frameIndex, levels=1
        )
        if stackFrames is not None:
            stackFrame = stackFrames[0]
            if "source" in stackFrame:
                source = stackFrame["source"]
                if "path" in source:
                    if "line" in stackFrame:
                        return (source["path"], stackFrame["line"])
        return ("", 0)

    def get_stdout(self):
        return self.dap_server.get_output("stdout")

    def get_console(self):
        return self.dap_server.get_output("console")

    def get_important(self):
        return self.dap_server.get_output("important")

    def collect_stdout(self, pattern: Optional[str] = None) -> str:
        return self.dap_server.collect_output("stdout", pattern=pattern)

    def collect_console(self, pattern: Optional[str] = None) -> str:
        return self.dap_server.collect_output("console", pattern=pattern)

    def collect_important(self, pattern: Optional[str] = None) -> str:
        return self.dap_server.collect_output("important", pattern=pattern)

    def get_local_as_int(self, name, threadId=None):
        value = self.dap_server.get_local_variable_value(name, threadId=threadId)
        # 'value' may have the variable value and summary.
        # Extract the variable value since summary can have nonnumeric characters.
        value = value.split(" ")[0]
        if value.startswith("0x"):
            return int(value, 16)
        elif value.startswith("0"):
            return int(value, 8)
        else:
            return int(value)

    def set_variable(self, varRef, name, value, id=None, is_hex: Optional[bool] = None):
        """Set a variable."""
        response = self.dap_server.request_setVariable(
            varRef, name, str(value), id=id, is_hex=is_hex
        )
        if response["success"]:
            invalidated_event = self.dap_server.wait_for_invalidated()
            self.assertEqual(invalidated_event["body"].get("areas"), ["variables"])
            memory_event = self.dap_server.wait_for_memory()
            self.assertEqual(
                memory_event["body"].get("memoryReference"),
                response["body"].get("memoryReference"),
            )
        return response

    def set_local(self, name, value, id=None, is_hex: Optional[bool] = None):
        """Set a top level local variable only."""
        # Get the locals scope reference dynamically
        locals_ref = self.get_locals_scope_reference()
        if locals_ref is None:
            return None
        return self.set_variable(locals_ref, name, str(value), id=id, is_hex=is_hex)

    def get_locals_scope_reference(self):
        """Get the variablesReference for the locals scope."""
        stackFrame = self.dap_server.get_stackFrame()
        if stackFrame is None:
            return None
        frameId = stackFrame["id"]
        scopes_response = self.dap_server.request_scopes(frameId)
        frame_scopes = scopes_response["body"]["scopes"]
        for scope in frame_scopes:
            if scope["name"] == "Locals":
                return scope["variablesReference"]
        return None

    def stepIn(
        self,
        threadId=None,
        targetId=None,
        waitForStop=True,
        granularity="statement",
    ):
        response = self.dap_server.request_stepIn(
            threadId=threadId, targetId=targetId, granularity=granularity
        )
        self.assertTrue(response["success"])
        if waitForStop:
            return self.dap_server.wait_for_stopped()
        return None

    def stepOver(
        self,
        threadId=None,
        waitForStop=True,
        granularity="statement",
    ):
        response = self.dap_server.request_next(
            threadId=threadId, granularity=granularity
        )
        self.assertTrue(
            response["success"], f"next request failed: response {response}"
        )
        if waitForStop:
            return self.dap_server.wait_for_stopped()
        return None

    def stepOut(self, threadId=None, waitForStop=True):
        self.dap_server.request_stepOut(threadId=threadId)
        if waitForStop:
            return self.dap_server.wait_for_stopped()
        return None

    def do_continue(self):  # `continue` is a keyword.
        resp = self.dap_server.request_continue()
        self.assertTrue(resp["success"], f"continue request failed: {resp}")

    def continue_to_next_stop(self):
        self.do_continue()
        return self.dap_server.wait_for_stopped()

    def continue_to_breakpoint(self, breakpoint_id: int):
        self.continue_to_breakpoints([breakpoint_id])

    def continue_to_breakpoints(self, breakpoint_ids: List[int]):
        self.do_continue()
        self.verify_breakpoint_hit(breakpoint_ids)

    def continue_to_exception_breakpoint(
        self, expected_description, expected_text=None
    ):
        self.do_continue()
        self.verify_stop_exception_info(expected_description, expected_text)

    def continue_to_exit(self, exitCode=0):
        self.do_continue()
        self.verify_process_exited(exitCode)

    def verify_process_exited(self, exitCode: int = 0):
        stopped_events = self.dap_server.wait_for_stopped()
        self.assertEqual(
            len(stopped_events), 1, "stopped_events = {}".format(stopped_events)
        )
        self.assertEqual(
            stopped_events[0]["event"], "exited", "make sure program ran to completion"
        )
        self.assertEqual(
            stopped_events[0]["body"]["exitCode"],
            exitCode,
            "exitCode == %i" % (exitCode),
        )

    def disassemble(self, threadId=None, frameIndex=None):
        stackFrames = self.get_stackFrames(
            threadId=threadId, startFrame=frameIndex, levels=1
        )
        self.assertIsNotNone(stackFrames)
        memoryReference = stackFrames[0]["instructionPointerReference"]
        self.assertIsNotNone(memoryReference)

        instructions = self.dap_server.request_disassemble(
            memoryReference=memoryReference
        )
        disassembled_instructions = {}
        for inst in instructions:
            disassembled_instructions[inst["address"]] = inst

        return disassembled_instructions, disassembled_instructions[memoryReference]

    def attach(
        self,
        *,
        client_features: Optional[dict[str, bool]] = None,
        disconnectAutomatically=True,
        sourceInitFile=False,
        **kwargs,
    ) -> int:
        """Attach to the process."""

        # Make sure we disconnect and terminate the DAP debug adapter even
        # if we throw an exception during the test case.
        def cleanup():
            if disconnectAutomatically:
                self.dap_server.request_disconnect(terminateDebuggee=True)
            self.dap_server.terminate()

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)
        # Initialize and launch the program
        self.dap_server.request_initialize(
            client_features=client_features, sourceInitFile=sourceInitFile
        )
        return self.dap_server.request_attach(**kwargs)

    def attach_and_configurationDone(
        self,
        **kwargs,
    ) -> Response:
        """Attach to the process."""
        attach_seq = self.attach(**kwargs)
        self.dap_server.wait_for_event(["initialized"])
        self.dap_server.request_configurationDone()
        return self.dap_server.receive_response(attach_seq)

    def launch(
        self,
        program: str,
        *,
        client_features: Optional[dict[str, bool]] = None,
        sourceInitFile=False,
        disconnectAutomatically=True,
        **kwargs,
    ) -> int:
        """Send launch request."""

        # Make sure we disconnect and terminate the DAP debug adapter,
        # if we throw an exception during the test case
        def cleanup():
            if disconnectAutomatically:
                self.dap_server.request_disconnect(terminateDebuggee=True)
            self.dap_server.terminate()

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Initialize and launch the program
        self.dap_server.request_initialize(
            client_features=client_features, sourceInitFile=sourceInitFile
        )
        return self.dap_server.request_launch(program, **kwargs)

    def launch_and_configurationDone(
        self,
        program: str,
        **kwargs,
    ) -> Response:
        """Send launch request."""
        launch_seq = self.launch(program, **kwargs)
        self.dap_server.wait_for_event(["initialized"])
        self.dap_server.request_configurationDone()
        return self.dap_server.receive_response(launch_seq)

    def build_and_launch(
        self,
        program,
        *,
        lldbDAPEnv: Optional[dict[str, str]] = None,
        **kwargs,
    ):
        """Build the default Makefile target, create the DAP debug adapter,
        and launch the process.
        """
        self.build_and_create_debug_adapter(lldbDAPEnv)
        self.assertTrue(os.path.exists(program), "executable must exist")

        return self.launch(program, **kwargs)

    def verify_configuration_done(self, expected_success=True):
        resp = self.dap_server.request_configurationDone()
        if expected_success:
            self.assertTrue(resp["success"])
        else:
            self.assertFalse(resp["success"])

    def getBuiltinDebugServerTool(self):
        # Tries to find simulation/lldb-server/gdbserver tool path.
        server_tool = None
        if lldbplatformutil.getPlatform() == "linux":
            server_tool = lldbgdbserverutils.get_lldb_server_exe()
            if server_tool is None:
                self.dap_server.request_disconnect(terminateDebuggee=True)
                self.assertIsNotNone(server_tool, "lldb-server not found.")
        elif lldbplatformutil.getPlatform() == "macosx":
            server_tool = lldbgdbserverutils.get_debugserver_exe()
            if server_tool is None:
                self.dap_server.request_disconnect(terminateDebuggee=True)
                self.assertIsNotNone(server_tool, "debugserver not found.")
        return server_tool

    def writeMemory(self, memoryReference, data=None, offset=0, allowPartial=False):
        # This function accepts data in decimal and hexadecimal format,
        # converts it to a Base64 string, and send it to the DAP,
        # which expects Base64 encoded data.
        encodedData = (
            ""
            if data is None
            else base64.b64encode(
                # (bit_length + 7 (rounding up to nearest byte) ) //8 = converts to bytes.
                data.to_bytes((data.bit_length() + 7) // 8, "little")
            ).decode()
        )
        response = self.dap_server.request_writeMemory(
            memoryReference, encodedData, offset=offset, allowPartial=allowPartial
        )
        if response["success"]:
            invalidated_event = self.dap_server.wait_for_invalidated()
            self.assertEqual(invalidated_event["body"].get("areas"), ["all"])
        return response



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class NavigateHiddenFrameTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        """Test going up/down a backtrace but we started in a hidden frame."""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Break here", lldb.SBFileSpec("main.cpp")
        )
        # up
        self.assertIn("__impl2", thread.selected_frame.GetFunctionName())
        self.expect("up")
        self.assertIn("__impl1", thread.selected_frame.GetFunctionName())
        self.expect("up")
        self.assertIn("__impl", thread.selected_frame.GetFunctionName())
        self.expect("up")
        self.assertIn("non_impl", thread.selected_frame.GetFunctionName())

        # Back down again.
        self.expect("down")
        self.assertIn("__impl", thread.selected_frame.GetFunctionName())
        self.expect("down")
        self.assertIn("__impl1", thread.selected_frame.GetFunctionName())
        self.expect("down")
        self.assertIn("__impl2", thread.selected_frame.GetFunctionName())



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.BroadcastEventByType(lldb.eBreakpointEventTypeInvalidType, True)
    obj.BroadcastEvent(lldb.SBEvent(), False)
    listener = lldb.SBListener("fuzz_testing")
    obj.AddInitialEventsToListener(listener, 0xFFFFFFFF)
    obj.AddInitialEventsToListener(listener, 0)
    obj.AddListener(listener, 0xFFFFFFFF)
    obj.AddListener(listener, 0)
    obj.GetName()
    obj.EventTypeHasListeners(0)
    obj.RemoveListener(listener, 0xFFFFFFFF)
    obj.RemoveListener(listener, 0)
    obj.Clear()



"""
Test that a frame provider can pass through all frames from its parent
StackFrameList while modifying function names.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import TestBase
from lldbsuite.test import lldbutil


class FrameProviderPassThroughPrefixTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # The frame list IDs used by 'bt --provider' match the descriptor IDs
    # returned by RegisterScriptedFrameProvider:
    # 0 = base unwinder, 1 = first provider, 2 = second provider, etc.
    UNWINDER_FRAME_LIST_ID = 0
    FIRST_PROVIDER_FRAME_LIST_ID = 1
    SECOND_PROVIDER_FRAME_LIST_ID = 2

    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.c"

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    def test_pass_through_with_prefix(self):
        """
        Test that a provider can read every frame from its parent list and
        return them with a prefixed function name.

        The call stack is main -> foo -> bar -> baz, with a breakpoint in
        baz. After registering the provider, every frame's function name
        should be prefixed with 'my_custom_'.
        """
        self.build()

        (target, process, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        # Verify the original backtrace: baz, bar, foo, main.
        expected_names = ["baz", "bar", "foo", "main"]
        for i, name in enumerate(expected_names):
            frame = thread.GetFrameAtIndex(i)
            self.assertEqual(
                frame.GetFunctionName(),
                name,
                f"Frame {i} should be '{name}' before provider",
            )

        original_frame_count = thread.GetNumFrames()
        self.assertGreaterEqual(
            original_frame_count, 4, "Should have at least 4 frames"
        )

        # Import and register the provider.
        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Frame count should be unchanged since we're passing through.
        new_frame_count = thread.GetNumFrames()
        self.assertEqual(
            new_frame_count,
            original_frame_count,
            "Frame count should be unchanged (pass-through provider)",
        )

        # Every frame should now have the 'my_custom_' prefix exactly once.
        prefix = "my_custom_"
        for i, name in enumerate(expected_names):
            frame = thread.GetFrameAtIndex(i)
            expected = prefix + name
            self.assertEqual(
                frame.GetFunctionName(),
                expected,
                f"Frame {i} should be '{expected}' after provider",
            )

    def test_bt_provider_shows_unwinder_frames(self):
        """
        Test that 'bt --provider 0' shows the base unwinder frames
        (without the provider prefix) even after a provider is registered.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )

        # 'bt --provider 0' should show the base unwinder frames without prefix.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider 0", result)
        self.assertTrue(result.Succeeded(), "bt --provider 0 should succeed")
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("baz", output)
        self.assertNotIn("my_custom_", output)

    def test_bt_provider_shows_provider_frames(self):
        """
        Test that 'bt --provider <id>' shows the provider's transformed frames
        with the prefix applied.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # 'bt --provider <id>' should show the provider's prefixed frames.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            f"bt --provider {self.FIRST_PROVIDER_FRAME_LIST_ID}", result
        )
        self.assertTrue(
            result.Succeeded(),
            f"bt --provider {self.FIRST_PROVIDER_FRAME_LIST_ID} should succeed",
        )
        output = result.GetOutput()
        self.assertIn("PrefixPassThroughProvider", output)
        self.assertIn("my_custom_baz", output)
        self.assertIn("my_custom_bar", output)
        self.assertIn("my_custom_foo", output)
        self.assertIn("my_custom_main", output)

    def test_bt_provider_range(self):
        """
        Test that 'bt --provider 0-<id>' shows both the base unwinder
        and provider frames sequentially.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # 'bt --provider 0-<id>' should show both sections.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            f"bt --provider 0-{self.FIRST_PROVIDER_FRAME_LIST_ID}", result
        )
        self.assertTrue(
            result.Succeeded(),
            f"bt --provider 0-{self.FIRST_PROVIDER_FRAME_LIST_ID} should succeed",
        )
        output = result.GetOutput()
        # Should contain both the base unwinder and the provider sections.
        self.assertIn("Base Unwinder", output)
        self.assertIn("PrefixPassThroughProvider", output)

    def test_bt_provider_range_not_starting_at_zero(self):
        """
        Test that 'bt --provider <id>-<id>' works when the range doesn't
        include the base unwinder (provider 0).
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Range that only includes the provider, not the base unwinder.
        fid = self.FIRST_PROVIDER_FRAME_LIST_ID
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            f"bt --provider {fid}-{fid}", result
        )
        self.assertTrue(
            result.Succeeded(),
            f"bt --provider {fid}-{fid} should succeed",
        )
        output = result.GetOutput()
        self.assertNotIn("Base Unwinder", output)
        self.assertIn("PrefixPassThroughProvider", output)
        self.assertIn("my_custom_baz", output)

    def test_bt_provider_range_with_to_separator(self):
        """
        Test that 'bt --provider 0 to <id>' works with the 'to' separator.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Use 'to' separator instead of '-'.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            f"bt --provider '0 to {self.FIRST_PROVIDER_FRAME_LIST_ID}'", result
        )
        self.assertTrue(
            result.Succeeded(),
            f"bt --provider '0 to {self.FIRST_PROVIDER_FRAME_LIST_ID}' should succeed",
        )
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("PrefixPassThroughProvider", output)

    def test_bt_provider_invalid_id(self):
        """
        Test that 'bt --provider <invalid>' fails with an error when the
        provider ID doesn't exist.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider 999", result)
        self.assertFalse(result.Succeeded(), "bt --provider 999 should fail")

    def test_bt_provider_invalid_range(self):
        """
        Test that 'bt --provider N-M' where N > M fails with an error
        about an invalid range.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider 5-2", result)
        self.assertFalse(result.Succeeded(), "bt --provider 5-2 should fail")
        self.assertIn("invalid provider range", result.GetError())

    def test_bt_provider_star_shows_all(self):
        """
        Test that 'bt --provider *' shows all providers including the
        base unwinder and any registered providers.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(error.Success(), f"Should register provider: {error}")

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider '*'", result)
        self.assertTrue(result.Succeeded(), "bt --provider '*' should succeed")
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("PrefixPassThroughProvider", output)

    def test_bt_provider_all_shows_all(self):
        """
        Test that 'bt --provider all' shows all providers including the
        base unwinder and any registered providers.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(error.Success(), f"Should register provider: {error}")

        target.RegisterScriptedFrameProvider(
            "frame_provider.UpperCasePassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(error.Success(), f"Should register upper provider: {error}")

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider all", result)
        self.assertTrue(result.Succeeded(), "bt --provider all should succeed")
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("PrefixPassThroughProvider", output)
        self.assertIn("UpperCasePassThroughProvider", output)

    def test_bt_provider_multiple_providers(self):
        """
        Test 'bt --provider' with two chained providers. Register
        PrefixPassThroughProvider (adds 'my_custom_' prefix) then
        UpperCasePassThroughProvider (upper-cases names). Verify each
        provider's view is correct and that ranges across providers work.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        # Register first provider: adds 'my_custom_' prefix.
        error = lldb.SBError()
        target.RegisterScriptedFrameProvider(
            "frame_provider.PrefixPassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(error.Success(), f"Should register prefix provider: {error}")

        # Register second provider: upper-cases everything.
        target.RegisterScriptedFrameProvider(
            "frame_provider.UpperCasePassThroughProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(error.Success(), f"Should register upper provider: {error}")

        # bt --provider 0: base unwinder, original names.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider 0", result)
        self.assertTrue(result.Succeeded())
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("baz", output)

        # bt --provider all: should show all providers.
        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand("bt --provider all", result)
        self.assertTrue(result.Succeeded())
        output = result.GetOutput()
        self.assertIn("Base Unwinder", output)
        self.assertIn("UpperCasePassThroughProvider", output)
        # UpperCase wraps Prefix (registration order is preserved), so the
        # outermost output should have fully upper-cased prefixed names.
        self.assertIn("MY_CUSTOM_BAZ", output)

    def test_bt_provider_star_from_within_provider(self):
        """
        Test that running 'bt --provider *' re-entrantly from within a
        scripted frame provider's get_frame_at_index does not deadlock
        or crash.

        BtProviderStarProvider runs 'bt --provider *' on its first
        get_frame_at_index call and stores the output, then passes through
        all frames with a 'reentrant_' prefix. We verify:
        1. The provider completes without hanging.
        2. Frame 0 has the 'reentrant_' prefix.
        """
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.BtProviderStarProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # Access frame 0 to trigger the provider (and the re-entrant bt call).
        # Only frame 0 is checked because the re-entrant HandleCommand resets
        # the selected frame on the unwinder list, which corrupts subsequent
        # frame lookups via inlined depth adjustments.
        frame = thread.GetFrameAtIndex(0)
        self.assertEqual(
            frame.GetFunctionName(),
            "reentrant_baz",
            "Frame 0 should be 'reentrant_baz' after provider",
        )

    def test_provider_receives_parent_frames(self):
        """
        Test that the provider's input_frames come from the parent
        StackFrameList, not from the provider's own output.

        The provider peeks at input_frames[0] when constructing frame 1.
        If that name already carries the 'my_custom_' prefix, the provider
        knows it was given its own output list and flags the error by
        inserting a 'danger_will_robinson_' prefix. The test asserts that
        'danger_will_robinson_' never
        appears, proving the provider received the bare parent list.
        """
        self.build()

        (target, process, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec(self.source)
        )

        # Import and register the validating provider.
        script_path = os.path.join(self.getSourceDir(), "frame_provider.py")
        self.runCmd("command script import " + script_path)

        error = lldb.SBError()
        provider_id = target.RegisterScriptedFrameProvider(
            "frame_provider.ValidatingPrefixProvider",
            lldb.SBStructuredData(),
            error,
        )
        self.assertTrue(
            error.Success(), f"Should register provider successfully: {error}"
        )
        self.assertNotEqual(provider_id, 0, "Provider ID should be non-zero")

        # No frame should contain 'danger_will_robinson_' — that would mean
        # the provider was handed its own output list instead of the parent.
        num_frames = thread.GetNumFrames()
        for i in range(num_frames):
            frame = thread.GetFrameAtIndex(i)
            actual = frame.GetFunctionName()
            self.assertFalse(
                actual.startswith("danger_will_robinson_"),
                f"Frame {i}: provider got its own output list "
                f"(expected bare parent frames, got '{actual}')",
            )



# RUN: env SUPPORT_LIB=%mlir_c_runner_utils \
# RUN:   %PYTHON %s | FileCheck %s

import ctypes
import numpy as np
import os
import sys

from mlir import ir
from mlir import runtime as rt

from mlir.dialects import sparse_tensor as st
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects.linalg.opdsl import lang as dsl

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)
from tools import sparsifier


@dsl.linalg_structured_op
def sddmm_dsl(
    A=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.K),
    B=dsl.TensorDef(dsl.T, dsl.S.K, dsl.S.N),
    S=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.N),
    C=dsl.TensorDef(dsl.T, dsl.S.M, dsl.S.N, output=True),
):
    C[dsl.D.m, dsl.D.n] += (
        S[dsl.D.m, dsl.D.n] * A[dsl.D.m, dsl.D.k] * B[dsl.D.k, dsl.D.n]
    )


def build_SDDMM(attr: st.EncodingAttr):
    """Build SDDMM kernel.

    This method generates a linalg op with for matrix multiplication using
    just the Python API. Effectively, a generic linalg op is constructed
    that computes C(i,j) += S(i,j) SUM_k A(i,k) B(k,j) for sparse S.
    """
    module = ir.Module.create()
    f64 = ir.F64Type.get()
    a = ir.RankedTensorType.get([8, 8], f64)
    b = ir.RankedTensorType.get([8, 8], f64)
    c = ir.RankedTensorType.get([8, 8], f64)
    s = ir.RankedTensorType.get([8, 8], f64, attr)
    arguments = [a, b, s, c]
    with ir.InsertionPoint(module.body):

        @func.FuncOp.from_py_func(*arguments)
        def sddmm(*args):
            return sddmm_dsl(args[0], args[1], args[2], outs=[args[3]])

    return module


def boilerplate(attr: st.EncodingAttr):
    """Returns boilerplate code for main driver."""
    return f"""
func.func @main(%a: tensor<8x8xf64>,
           %b: tensor<8x8xf64>,
           %c: tensor<8x8xf64>) -> tensor<8x8xf64> attributes {{ llvm.emit_c_interface }} {{
  %t = arith.constant sparse<[[0,0], [0,2], [4,1]], [1.0, 2.0, 3.0]> : tensor<8x8xf64>
  %s = sparse_tensor.convert %t : tensor<8x8xf64> to tensor<8x8xf64, {attr}>
  %0 = call @sddmm(%a, %b, %s, %c) : (tensor<8x8xf64>,
                                      tensor<8x8xf64>,
                                      tensor<8x8xf64, {attr}>,
                                      tensor<8x8xf64>) -> tensor<8x8xf64>
  return %0 : tensor<8x8xf64>
}}
"""


def build_compile_and_run_SDDMMM(attr: st.EncodingAttr, compiler):
    # Build.
    module = build_SDDMM(attr)
    func = str(module.operation.regions[0].blocks[0].operations[0].operation)
    module = ir.Module.parse(func + boilerplate(attr))

    # Compile.
    engine = compiler.compile_and_jit(module)

    # Set up numpy input and buffer for output.
    a = np.array(
        [
            [1.1, 2.1, 3.1, 4.1, 5.1, 6.1, 7.1, 8.1],
            [1.2, 2.2, 3.2, 4.2, 5.2, 6.2, 7.2, 8.2],
            [1.3, 2.3, 3.3, 4.3, 5.3, 6.3, 7.3, 8.3],
            [1.4, 2.4, 3.4, 4.4, 5.4, 6.4, 7.4, 8.4],
            [1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5],
            [1.6, 2.6, 3.6, 4.6, 5.6, 6.6, 7.6, 8.6],
            [1.7, 2.7, 3.7, 4.7, 5.7, 6.7, 7.7, 8.7],
            [1.8, 2.8, 3.8, 4.8, 5.8, 6.8, 7.8, 8.8],
        ],
        np.float64,
    )
    b = np.ones((8, 8), np.float64)
    c = np.zeros((8, 8), np.float64)

    mem_a = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(a)))
    mem_b = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(b)))
    mem_c = ctypes.pointer(ctypes.pointer(rt.get_ranked_memref_descriptor(c)))

    # Allocate a MemRefDescriptor to receive the output tensor.
    # The buffer itself is allocated inside the MLIR code generation.
    ref_out = rt.make_nd_memref_descriptor(2, ctypes.c_double)()
    mem_out = ctypes.pointer(ctypes.pointer(ref_out))

    # Invoke the kernel and get numpy output.
    # Built-in bufferization uses in-out buffers.
    engine.invoke("main", mem_out, mem_a, mem_b, mem_c)

    # Sanity check on computed result. Only a few elements
    # are sampled from the full dense matrix multiplication.
    full_matmul = np.matmul(a, b)
    expected = np.zeros((8, 8), np.float64)
    expected[0, 0] = 1.0 * full_matmul[0, 0]
    expected[0, 2] = 2.0 * full_matmul[0, 2]
    expected[4, 1] = 3.0 * full_matmul[4, 1]
    c = rt.ranked_memref_to_numpy(mem_out[0])
    if np.allclose(c, expected):
        pass
    else:
        quit(f"FAILURE")


def main():
    support_lib = os.getenv("SUPPORT_LIB")
    assert support_lib is not None, "SUPPORT_LIB is undefined"
    if not os.path.exists(support_lib):
        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), support_lib)

    # CHECK-LABEL: TEST: testSDDMMM
    print("\nTEST: testSDDMMM")
    count = 0
    with ir.Context() as ctx, ir.Location.unknown():
        # Loop over various ways to compile and annotate the SDDMM kernel with
        # a *single* sparse tensor. Note that we deliberate do not exhaustively
        # search the full state space to reduce runtime of the test. It is
        # straightforward to adapt the code below to explore more combinations.
        # For these simple orderings, dim2lvl and lvl2dim are the same.
        builder = st.EncodingAttr.build_level_type
        fmt = st.LevelFormat
        prop = st.LevelProperty
        levels = [
            [builder(fmt.compressed, [prop.non_unique]), builder(fmt.singleton)],
            [builder(fmt.dense), builder(fmt.dense)],
            [builder(fmt.dense), builder(fmt.compressed)],
            [builder(fmt.compressed), builder(fmt.dense)],
            [builder(fmt.compressed), builder(fmt.compressed)],
        ]
        orderings = [
            ir.AffineMap.get_permutation([0, 1]),
            ir.AffineMap.get_permutation([1, 0]),
        ]
        for level in levels:
            for ordering in orderings:
                for pwidth in [32]:
                    for iwidth in [32]:
                        for e in [True]:
                            attr = st.EncodingAttr.get(
                                level, ordering, ordering, pwidth, iwidth
                            )
                            opt = f"parallelization-strategy=none"
                            compiler = sparsifier.Sparsifier(
                                extras="",
                                options=opt,
                                opt_level=0,
                                shared_libs=[support_lib],
                            )
                            build_compile_and_run_SDDMMM(attr, compiler)
                            count = count + 1
    # CHECK: Passed 10 tests
    print("Passed ", count, "tests")


if __name__ == "__main__":
    main()



"""
Test the session save feature
"""
import os
import tempfile

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SessionSaveTestCase(TestBase):
    def raw_transcript_builder(self, cmd, res):
        raw = "(lldb) " + cmd + "\n"
        if res.GetOutputSize():
            raw += res.GetOutput()
        if res.GetErrorSize():
            raw += res.GetError()
        return raw

    @no_debug_info_test
    def test_session_save(self):
        raw = ""
        interpreter = self.dbg.GetCommandInterpreter()

        # Make sure "save-transcript" is on, so that all the following setings
        # and commands are saved into the trasncript. Note that this cannot be
        # a part of the `settings`, because this command itself won't be saved
        # into the transcript.
        self.runCmd("settings set interpreter.save-transcript true")

        settings = [
            "settings set interpreter.echo-commands true",
            "settings set interpreter.echo-comment-commands true",
            "settings set interpreter.stop-command-source-on-error false",
            "settings set interpreter.open-transcript-in-editor false",
        ]

        for setting in settings:
            interpreter.HandleCommand(setting, lldb.SBCommandReturnObject())

        inputs = [
            "# This is a comment",  # Comment
            "help session",  # Valid command
            "Lorem ipsum",  # Invalid command
        ]

        for cmd in inputs:
            res = lldb.SBCommandReturnObject()
            interpreter.HandleCommand(cmd, res)
            raw += self.raw_transcript_builder(cmd, res)

        self.assertTrue(interpreter.HasCommands())
        self.assertNotEqual(len(raw), 0)

        # Check for error
        cmd = "session save /root/file"
        interpreter.HandleCommand(cmd, res)
        self.assertFalse(res.Succeeded())
        raw += self.raw_transcript_builder(cmd, res)

        output_file = self.getBuildArtifact('my-session')

        res = lldb.SBCommandReturnObject()
        interpreter.HandleCommand("session save " + output_file, res)
        self.assertTrue(res.Succeeded())
        raw += self.raw_transcript_builder(cmd, res)

        with open(output_file, "r") as file:
            content = file.read()
            # Exclude last line, since session won't record it's own output
            lines = raw.splitlines()[:-1]
            for line in lines:
                self.assertIn(line, content)

        td = tempfile.TemporaryDirectory()
        res = lldb.SBCommandReturnObject()
        interpreter.HandleCommand(
            "settings set interpreter.save-session-directory " + td.name, res
        )
        self.assertTrue(res.Succeeded())

        res = lldb.SBCommandReturnObject()
        interpreter.HandleCommand("session save", res)
        self.assertTrue(res.Succeeded())
        raw += self.raw_transcript_builder(cmd, res)
        # Also check that we don't print an error message about an empty transcript.
        self.assertNotIn("interpreter.save-transcript is set to false", res.GetError())

        with open(os.path.join(td.name, os.listdir(td.name)[0]), "r") as file:
            content = file.read()
            # Exclude last line, since session won't record it's own output
            lines = raw.splitlines()[:-1]
            for line in lines:
                self.assertIn(line, content)

    @no_debug_info_test
    def test_session_save_no_transcript_warning(self):
        interpreter = self.dbg.GetCommandInterpreter()

        self.runCmd("settings set interpreter.save-transcript false")

        # These commands won't be saved, so are arbitrary.
        commands = [
            "settings set interpreter.open-transcript-in-editor false",
            "p 1",
            "settings set interpreter.save-session-on-quit true",
            "fr v",
            "settings set interpreter.echo-comment-commands true",
        ]

        for command in commands:
            res = lldb.SBCommandReturnObject()
            interpreter.HandleCommand(command, res)

        output_file = self.getBuildArtifact("my-session")

        res = lldb.SBCommandReturnObject()
        interpreter.HandleCommand("session save " + output_file, res)
        self.assertTrue(res.Succeeded())
        # We should warn about the setting being false.
        self.assertIn("interpreter.save-transcript is set to false", res.GetError())
        self.assertTrue(
            os.path.getsize(output_file) == 0,
            "Output file should be empty since we didn't save the transcript.",
        )

    @no_debug_info_test
    def test_session_save_on_quit(self):
        raw = ""
        interpreter = self.dbg.GetCommandInterpreter()

        # Make sure "save-transcript" is on, so that all the following setings
        # and commands are saved into the trasncript. Note that this cannot be
        # a part of the `settings`, because this command itself won't be saved
        # into the transcript.
        self.runCmd("settings set interpreter.save-transcript true")

        td = tempfile.TemporaryDirectory()

        settings = [
            "settings set interpreter.echo-commands true",
            "settings set interpreter.echo-comment-commands true",
            "settings set interpreter.stop-command-source-on-error false",
            "settings set interpreter.save-session-on-quit true",
            "settings set interpreter.save-session-directory " + td.name,
            "settings set interpreter.open-transcript-in-editor false",
        ]

        for setting in settings:
            res = lldb.SBCommandReturnObject()
            interpreter.HandleCommand(setting, res)
            raw += self.raw_transcript_builder(setting, res)

        self.dbg.Destroy(self.dbg)

        with open(os.path.join(td.name, os.listdir(td.name)[0]), "r") as file:
            content = file.read()
            # Exclude last line, since session won't record it's own output
            lines = raw.splitlines()[:-1]
            for line in lines:
                self.assertIn(line, content)



"""
Tests that C++ member and static variables are available where they should be.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPThisTestCase(TestBase):
    # rdar://problem/9962849
    @expectedFailureAll(
        compiler="gcc",
        bugnumber="llvm.org/pr15439 The 'this' pointer isn't available during expression evaluation when stopped in an inlined member function",
    )
    @expectedFailureAll(
        compiler="icc",
        bugnumber="ICC doesn't emit correct DWARF inline debug info for inlined member functions.",
    )
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24489: Name lookup not working correctly on Windows",
    )
    def test_with_run_command(self):
        """Test that the appropriate member variables are available when stopped in C++ static, inline, and const methods"""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        self.set_breakpoint(line_number("main.cpp", "// breakpoint 1"))
        self.set_breakpoint(line_number("main.cpp", "// breakpoint 2"))
        self.set_breakpoint(line_number("main.cpp", "// breakpoint 3"))
        self.set_breakpoint(line_number("main.cpp", "// breakpoint 4"))

        self.runCmd("process launch", RUN_SUCCEEDED)

        self.expect("expression -- m_a = 2", startstr="(int) $0 = 2")

        self.runCmd("process continue")

        self.expect(
            "expression -- m_a = 2",
            error=True,
            substrs=[
                "cannot assign to non-static data member within const member function",
                "Possibly trying to mutate object in a const context. Try running the expression with: expression --c++-ignore-context-qualifiers -- m_a = 2",
            ],
        )

        self.expect("expression -- (int)getpid(); m_a", startstr="(const int) $1 = 3")
        self.expect(
            "expression --c++-ignore-context-qualifiers -- m_a = 2",
            startstr="(int) $2 = 2",
        )

        self.runCmd("process continue")

        self.expect("expression -- s_a", startstr="(int) $3 = 5")

        self.runCmd("process continue")

        self.expect("expression -- m_a", startstr="(int) $4 = 2")

    def set_breakpoint(self, line):
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", line, num_expected_locations=1, loc_exact=False
        )



"""Test that the Objective-C syntax for dictionary/array literals and indexing works"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCNewSyntaxTest(TestBase):
    def target(self):
        return self._target

    def runToBreakpoint(self):
        self.build()
        self._target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint 0 here.", lldb.SBFileSpec("main.m", False)
        )

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)



"""Test that a dSYM can be found when a binary is in a bundle hnd has dots in the filename."""


import os.path
from time import sleep

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


exe_name = "find-bundle-with-dots-in-fn"  # must match Makefile


class BundleWithDotInFilenameTestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.source = "main.c"

    def tearDown(self):
        # Destroy process before TestBase.tearDown()
        self.dbg.GetSelectedTarget().GetProcess().Destroy()

        # Call super's tearDown().
        TestBase.tearDown(self)

    @skipIfRemote
    @skipUnlessDarwin
    # This test is explicitly a dSYM test, it doesn't need to run for any other config.
    @skipIf(debug_info=no_match(["dsym"]))
    def test_attach_and_check_dsyms(self):
        """Test attach to binary, see if the bundle dSYM is found"""
        exe = self.getBuildArtifact(exe_name)
        self.build()
        os.chdir(self.getBuildDir())

        # Use a file as a synchronization point between test and inferior.
        pid_file_path = lldbutil.append_to_process_working_directory(
            self, "token_pid_%d" % (int(os.getpid()))
        )
        self.addTearDownHook(
            lambda: self.run_platform_command("rm %s" % (pid_file_path))
        )

        popen = self.spawnSubprocess(exe, [pid_file_path])

        # Wait for the inferior to start up, dlopen a bundle, remove the bundle it linked in
        pid = lldbutil.wait_for_file_on_target(self, pid_file_path)

        # Since the library that was dlopen()'ed is now removed, lldb will need to find the
        # binary & dSYM via target.exec-search-paths
        settings_str = (
            "settings set target.exec-search-paths "
            + self.get_process_working_directory()
            + "/hide.app"
        )
        self.runCmd(settings_str)

        self.runCmd("process attach -p " + str(popen.pid))

        target = self.dbg.GetSelectedTarget()
        self.assertTrue(
            target.IsValid(), "Should have a valid Target after attaching to process"
        )

        setup_complete = target.FindFirstGlobalVariable("setup_is_complete")
        self.assertEqual(
            setup_complete.GetValueAsUnsigned(),
            1,
            "Check that inferior process has completed setup",
        )

        # Find the bundle module, see if we found the dSYM too (they're both in "hide.app")
        i = 0
        while i < target.GetNumModules():
            mod = target.GetModuleAtIndex(i)
            if mod.GetFileSpec().GetFilename() == "com.apple.sbd":
                dsym_name = mod.GetSymbolFileSpec().GetFilename()
                self.assertEqual(
                    dsym_name,
                    "com.apple.sbd",
                    "Check that we found the dSYM for the bundle that was loaded",
                )
            i = i + 1
        os.chdir(self.getSourceDir())



"""
Test lldb data formatter for std::unique_ptr.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    TEST_WITH_PDB_DEBUG_INFO = True
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        """Test `frame variable` output for `std::unique_ptr` types."""

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        valobj = self.expect_var_path(
            "up_empty",
            summary="nullptr",
            children=[ValueCheck(name="pointer")],
        )
        self.assertEqual(
            valobj.child[0].GetValueAsUnsigned(lldb.LLDB_INVALID_ADDRESS), 0
        )

        self.expect(
            "frame variable *up_empty", substrs=["(int) *up_empty = <parent is NULL>"]
        )

        valobj = self.expect_var_path(
            "up_int",
            summary="10",
            children=[ValueCheck(name="pointer")],
        )
        self.assertNotEqual(valobj.child[0].unsigned, 0)

        valobj = self.expect_var_path(
            "up_int_ref",
            summary="10",
            children=[ValueCheck(name="pointer")],
        )
        self.assertNotEqual(valobj.child[0].unsigned, 0)

        valobj = self.expect_var_path(
            "up_int_ref_ref",
            summary="10",
            children=[ValueCheck(name="pointer")],
        )
        self.assertNotEqual(valobj.child[0].unsigned, 0)

        valobj = self.expect_var_path(
            "up_str",
            summary='"hello"',
            children=[ValueCheck(name="pointer", summary='"hello"')],
        )

        valobj = self.expect_var_path("up_user")
        self.assertRegex(valobj.summary, "^User @ 0x0*[1-9a-f][0-9a-f]+$")
        self.assertNotEqual(valobj.child[0].unsigned, 0)

        valobj = self.expect_var_path(
            "*up_user",
            type="User",
            children=[
                ValueCheck(name="id", value="30"),
                ValueCheck(name="name", summary='"steph"'),
            ],
        )
        self.assertEqual(str(valobj), '(User) *pointer = (id = 30, name = "steph")')

        valobj = self.expect_var_path(
            "up_non_empty_deleter",
            type="std::unique_ptr<int, NonEmptyIntDeleter>",
            summary="1234",
            children=[
                ValueCheck(name="pointer"),
                ValueCheck(
                    name="deleter", children=[ValueCheck(name="dummy_", value="9999")]
                ),
            ],
        )
        self.assertNotEqual(valobj.child[0].unsigned, 0)

        self.expect_var_path("up_user->id", type="int", value="30")
        self.expect_var_path(
            "up_user->name",
            type=(
                "std::basic_string<char, std::char_traits<char>, std::allocator<char>>"
                if self.getDebugInfo() == "pdb"
                else "std::string"
            ),
            summary='"steph"',
        )

        self.runCmd("settings set target.experimental.use-DIL true")
        self.expect_var_path("ptr_node->value", value="1")
        self.expect_var_path("ptr_node->next->value", value="2")
        self.expect_var_path("(*ptr_node).value", value="1")
        self.expect_var_path("(*(*ptr_node).next).value", value="2")

    @add_test_categories(["libstdcxx"])
    def test_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()

    @add_test_categories(["msvcstl"])
    def test_msvcstl(self):
        # No flags, because the "msvcstl" category checks that the MSVC STL is used by default.
        self.build()
        self.do_test()

    def do_test_recursive_unique_ptr(self):
        # Tests that LLDB can handle when we have a loop in the unique_ptr
        # reference chain and that it correctly handles the different options
        # for the frame variable command in this case.
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_source_regexp(self, "Set break point at this line.")
        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect("frame variable f1->next", substrs=["next = NodeU @"])
        self.expect(
            "frame variable --ptr-depth=1 f1->next",
            substrs=["next = NodeU @", "value = 2"],
        )
        self.expect(
            "frame variable --ptr-depth=2 f1->next",
            substrs=["next = NodeU @", "value = 1", "value = 2"],
        )

        frame = self.frame()
        self.assertTrue(frame.IsValid())
        self.assertEqual(
            2,
            frame.GetValueForVariablePath("f1->next.object.value").GetValueAsUnsigned(),
        )
        self.assertEqual(
            2, frame.GetValueForVariablePath("f1->next->value").GetValueAsUnsigned()
        )
        self.assertEqual(
            1,
            frame.GetValueForVariablePath(
                "f1->next.object.next.obj.value"
            ).GetValueAsUnsigned(),
        )
        self.assertEqual(
            1,
            frame.GetValueForVariablePath("f1->next->next->value").GetValueAsUnsigned(),
        )

    @add_test_categories(["libstdcxx"])
    def test_recursive_unique_ptr_libstdcxx(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test_recursive_unique_ptr()

    @add_test_categories(["libc++"])
    def test_recursive_unique_ptr_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test_recursive_unique_ptr()

    @add_test_categories(["msvcstl"])
    def test_recursive_unique_ptr_msvcstl(self):
        self.build()
        self.do_test_recursive_unique_ptr()



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase


class TestDAP_launch_stopOnEntry(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the default launch of a simple program that stops at the
    entry point instead of continuing.
    """

    def test(self):
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, stopOnEntry=True)
        self.dap_server.request_configurationDone()
        self.dap_server.wait_for_stopped()
        self.assertTrue(
            len(self.dap_server.thread_stop_reasons) > 0,
            "expected stopped event during launch",
        )
        for _, body in self.dap_server.thread_stop_reasons.items():
            if "reason" in body:
                reason = body["reason"]
                self.assertNotEqual(
                    reason, "breakpoint", 'verify stop isn\'t "main" breakpoint'
                )



"""
Test SBDebugger.InterruptRequested and SBCommandInterpreter.WasInterrupted.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
import threading
import os


class TestDebuggerInterruption(TestBase):
    """This test runs a command that starts up, rendevous with the test thread
       using threading barriers, then checks whether it has been interrupted.

    The command's first argument is either 'interp' or 'debugger', to test
    InterruptRequested and WasInterrupted respectively.

    The command has two modes, interrupt and check, the former is the one that
    waits for an interrupt.  Then latter just returns whether an interrupt was
    requested.  We use the latter to make sure we took down the flag correctly."""

    NO_DEBUG_INFO_TESTCASE = True

    class CommandRunner(threading.Thread):
        """This class is for running a command, and for making a thread to run the command on.
        It gets passed the test it is working on behalf of, and most of the important
        objects come from the test."""

        def __init__(self, test):
            super().__init__()
            self.test = test

        def rendevous(self):
            # We smuggle out barriers and event to the runner thread using thread local data:
            import interruptible

            interruptible.local_data = interruptible.BarrierContainer(
                self.test.before_interrupt_barrier,
                self.test.after_interrupt_barrier,
                self.test.event,
            )

    class DirectCommandRunner(CommandRunner):
        """ "This version runs a single command using HandleCommand."""

        def __init__(self, test, command):
            super().__init__(test)
            self.command = command

        def run(self):
            self.rendevous()
            result = self.test.dbg.GetCommandInterpreter().HandleCommand(
                self.command, self.test.result
            )
            if self.test.result_barrier:
                self.test.result_barrier.wait()

    class CommandInterpreterRunner(CommandRunner):
        """This version runs the CommandInterpreter and feeds the command to it."""

        def __init__(self, test):
            super().__init__(test)

        def run(self):
            self.rendevous()

            test = self.test

            # We will use files for debugger input and output:

            # First write down the command:
            with open(test.getBuildArtifact(test.in_filename), "w") as f:
                f.write(f"{test.command}\n")

            # Now set the debugger's stdout & stdin to our files, and run
            # the CommandInterpreter:
            with open(test.out_filename, "w") as outf, open(
                test.in_filename, "r"
            ) as inf:
                outsbf = lldb.SBFile(outf.fileno(), "w", False)
                orig_outf = test.dbg.GetOutputFile()
                error = test.dbg.SetOutputFile(outsbf)
                test.assertSuccess(error, "Could not set outfile")

                insbf = lldb.SBFile(inf.fileno(), "r", False)
                orig_inf = test.dbg.GetOutputFile()
                error = test.dbg.SetInputFile(insbf)
                test.assertSuccess(error, "Could not set infile")

                options = lldb.SBCommandInterpreterRunOptions()
                options.SetPrintResults(True)
                options.SetEchoCommands(False)

                test.dbg.RunCommandInterpreter(True, False, options, 0, False, False)
                test.dbg.GetOutputFile().Flush()

                error = test.dbg.SetOutputFile(orig_outf)
                test.assertSuccess(error, "Restored outfile")
                test.dbg.SetInputFile(orig_inf)
                test.assertSuccess(error, "Restored infile")

    def command_setup(self, args):
        """Insert our command, if needed.  Then set up event and barriers if needed.
        Then return the command to run."""

        self.interp = self.dbg.GetCommandInterpreter()
        self.command_name = "interruptible_command"
        self.cmd_result = lldb.SBCommandReturnObject()

        if not "check" in args:
            self.event = threading.Event()
            self.result_barrier = threading.Barrier(2, timeout=10)
            self.before_interrupt_barrier = threading.Barrier(2, timeout=10)
            self.after_interrupt_barrier = threading.Barrier(2, timeout=10)
        else:
            self.event = None
            self.result_barrier = None
            self.before_interrupt_barrier = None
            self.after_interrupt_barrier = None

        if not self.interp.UserCommandExists(self.command_name):
            # Make the command we're going to use - it spins calling WasInterrupted:
            cmd_filename = "interruptible"
            cmd_filename = os.path.join(self.getSourceDir(), "interruptible.py")
            self.runCmd(f"command script import {cmd_filename}")
            cmd_string = f"command script add {self.command_name} --class interruptible.WelcomeCommand"
            self.runCmd(cmd_string)

        if len(args) == 0:
            command = self.command_name
        else:
            command = self.command_name + " " + args
        return command

    def run_single_command(self, command):
        # Now start up a thread to run the command:
        self.result.Clear()
        self.runner = TestDebuggerInterruption.DirectCommandRunner(self, command)
        self.runner.start()

    def start_command_interp(self):
        self.runner = TestDebuggerInterruption.CommandInterpreterRunner(self)
        self.runner.start()

    def check_text(self, result_text, interrupted):
        if interrupted:
            self.assertIn(
                "Command was interrupted", result_text, "Got the interrupted message"
            )
        else:
            self.assertIn(
                "Command was not interrupted",
                result_text,
                "Got the not interrupted message",
            )

    def gather_output(self):
        # Now wait for the interrupt to interrupt the command:
        self.runner.join(10.0)
        finished = not self.runner.is_alive()
        # Don't leave the runner thread stranded if the interrupt didn't work.
        if not finished:
            self.event.set()
            self.runner.join(10.0)

        self.assertTrue(finished, "We did finish the command")

    def check_result(self, interrupted=True):
        self.gather_output()
        self.check_text(self.result.GetOutput(), interrupted)

    def check_result_output(self, interrupted=True):
        self.gather_output()
        buffer = ""
        # Okay, now open the file for reading, and read.
        with open(self.out_filename, "r") as f:
            buffer = f.read()

        self.assertNotEqual(len(buffer), 0, "No command data")
        self.check_text(buffer, interrupted)

    def debugger_interrupt_test(self, use_interrupt_requested):
        """Test that debugger interruption interrupts a command
        running directly through HandleCommand.
        If use_interrupt_requested is true, we'll check that API,
        otherwise we'll check WasInterrupted.  They should both do
        the same thing."""

        if use_interrupt_requested:
            command = self.command_setup("debugger")
        else:
            command = self.command_setup("interp")

        self.result = lldb.SBCommandReturnObject()
        self.run_single_command(command)

        # Okay now wait till the command has gotten started to issue the interrupt:
        self.before_interrupt_barrier.wait()
        # I'm going to do it twice here to test that it works as a counter:
        self.dbg.RequestInterrupt()
        self.dbg.RequestInterrupt()

        def cleanup():
            self.dbg.CancelInterruptRequest()

        self.addTearDownHook(cleanup)
        # Okay, now set both sides going:
        self.after_interrupt_barrier.wait()

        # Check that the command was indeed interrupted.  First rendevous
        # after the runner thread had a chance to execute the command:
        self.result_barrier.wait()
        self.assertTrue(self.result.Succeeded(), "Our command succeeded")
        result_output = self.result.GetOutput()
        self.check_result(True)

        # Do it again to make sure that the counter is counting:
        self.dbg.CancelInterruptRequest()
        command = self.command_setup("debugger")
        self.run_single_command(command)

        # This time we won't even get to run the command, since HandleCommand
        # checks for the interrupt state on entry, so we don't wait on the command
        # barriers.
        self.result_barrier.wait()

        # Again check that we were
        self.assertFalse(self.result.Succeeded(), "Our command was not allowed to run")
        error_output = self.result.GetError()
        self.assertIn(
            "... Interrupted", error_output, "Command was cut short by interrupt"
        )

        # Now take down the flag, and make sure that we aren't interrupted:
        self.dbg.CancelInterruptRequest()

        # Now make sure that we really did take down the flag:
        command = self.command_setup("debugger check")
        self.run_single_command(command)
        result_output = self.result.GetOutput()
        self.check_result(False)

    def test_debugger_interrupt_use_dbg(self):
        self.debugger_interrupt_test(True)

    def test_debugger_interrupt_use_interp(self):
        self.debugger_interrupt_test(False)

    def test_interp_doesnt_interrupt_debugger(self):
        """Test that interpreter interruption does not interrupt a command
        running directly through HandleCommand.
        If use_interrupt_requested is true, we'll check that API,
        otherwise we'll check WasInterrupted.  They should both do
        the same thing."""

        command = self.command_setup("debugger poll")

        self.result = lldb.SBCommandReturnObject()
        self.run_single_command(command)

        # Now raise the debugger interrupt flag.  It will also interrupt the command:
        self.before_interrupt_barrier.wait()
        self.dbg.GetCommandInterpreter().InterruptCommand()
        self.after_interrupt_barrier.wait()

        # Check that the command was indeed interrupted:
        self.result_barrier.wait()
        self.assertTrue(self.result.Succeeded(), "Our command succeeded")
        result_output = self.result.GetOutput()
        self.check_result(False)

    def interruptible_command_test(self, use_interrupt_requested):
        """Test that interpreter interruption interrupts a command
        running in the RunCommandInterpreter loop.
        If use_interrupt_requested is true, we'll check that API,
        otherwise we'll check WasInterrupted.  They should both do
        the same thing."""

        self.out_filename = self.getBuildArtifact("output")
        self.in_filename = self.getBuildArtifact("input")
        # We're going to overwrite the input file, but we
        # don't want data accumulating in the output file.

        if os.path.exists(self.out_filename):
            os.unlink(self.out_filename)

        # You should be able to use either check method interchangeably:
        if use_interrupt_requested:
            self.command = self.command_setup("debugger") + "\n"
        else:
            self.command = self.command_setup("interp") + "\n"

        self.start_command_interp()

        # Now give the interpreter a chance to run this command up
        # to the first barrier
        self.before_interrupt_barrier.wait()
        # Then issue the interrupt:
        sent_interrupt = self.dbg.GetCommandInterpreter().InterruptCommand()
        self.assertTrue(sent_interrupt, "Did send command interrupt.")
        # Now give the command a chance to finish:
        self.after_interrupt_barrier.wait()

        self.check_result_output(True)

        os.unlink(self.out_filename)

        # Now send the check command, and make sure the flag is now down.
        self.command = self.command_setup("interp check") + "\n"
        self.start_command_interp()

        self.check_result_output(False)

    def test_interruptible_command_check_dbg(self):
        self.interruptible_command_test(True)

    def test_interruptible_command_check_interp(self):
        self.interruptible_command_test(False)

    def test_debugger_doesnt_interrupt_command(self):
        """Test that debugger interruption doesn't interrupt a command
        running in the RunCommandInterpreter loop."""

        self.out_filename = self.getBuildArtifact("output")
        self.in_filename = self.getBuildArtifact("input")
        # We're going to overwrite the input file, but we
        # don't want data accumulating in the output file.

        if os.path.exists(self.out_filename):
            os.unlink(self.out_filename)

        self.command = self.command_setup("interp poll") + "\n"

        self.start_command_interp()

        self.before_interrupt_barrier.wait()
        self.dbg.RequestInterrupt()

        def cleanup():
            self.dbg.CancelInterruptRequest()

        self.addTearDownHook(cleanup)
        self.after_interrupt_barrier.wait()

        self.check_result_output(False)

        os.unlink(self.out_filename)



"""
Test the redirection of stdio.
There are three ways to launch the debuggee
internalConsole, integratedTerminal and externalTerminal.

For the three configurations, we test if we can read data
from environments, stdin and cli arguments.

NOTE: The testcases do not include all possible configurations of
consoles, environments, stdin and cli arguments.
"""

from abc import abstractmethod
import lldbdap_testcase
from tempfile import NamedTemporaryFile


class DAP_launchIO(lldbdap_testcase.DAPTestCaseBase):
    """The class holds the implementation different ways to redirect the debuggee I/O streams
    which is configurable from the Derived classes.

    Depending on the console type the output will be in different places.
    It also provides two abstract functions `_get_debuggee_stdout` and `_get_debuggee_stderr`
    that provides the debuggee stdout and stderr.
    """

    def all_redirection(self, console: str, with_args: bool = False):
        """Test all standard io redirection."""
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")
        input_text = "from stdin with redirection"
        args_text = "string from argv"
        program_args = [args_text] if with_args else None

        with NamedTemporaryFile("wt") as stdin, NamedTemporaryFile(
            "rt"
        ) as stdout, NamedTemporaryFile("rt") as stderr:
            stdin.write(input_text)
            stdin.flush()
            self.launch_and_configurationDone(
                program,
                stdio=[stdin.name, stdout.name, stderr.name],
                console=console,
                args=program_args,
            )
            self.verify_process_exited()

            all_stdout = stdout.read()
            all_stderr = stderr.read()

            if with_args:
                self.assertEqual(f"[STDOUT][FROM_ARGV]: {args_text}", all_stdout)
                self.assertEqual(f"[STDERR][FROM_ARGV]: {args_text}", all_stderr)

                self.assertNotIn(f"[STDOUT][FROM_ARGV]: {args_text}", all_stderr)
                self.assertNotIn(f"[STDERR][FROM_ARGV]: {args_text}", all_stdout)

            else:
                self.assertEqual(f"[STDOUT][FROM_STDIN]: {input_text}", all_stdout)
                self.assertEqual(f"[STDERR][FROM_STDIN]: {input_text}", all_stderr)

                self.assertNotIn(f"[STDERR][FROM_STDIN]: {input_text}", all_stdout)
                self.assertNotIn(f"[STDOUT][FROM_STDIN]: {input_text}", all_stderr)

    def stdin_redirection(self, console: str, with_args: bool = False):
        """Test only stdin redirection."""
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")
        input_text = "string from stdin"
        args_text = "string from argv"
        program_args = [args_text] if with_args else None

        with NamedTemporaryFile("w+t") as stdin:
            stdin.write(input_text)
            stdin.flush()
            self.launch_and_configurationDone(
                program, stdio=[stdin.name], console=console, args=program_args
            )
            self.verify_process_exited()

            stdout_text = self._get_debuggee_stdout()
            stderr_text = self._get_debuggee_stderr()

            if with_args:
                self.assertIn(f"[STDOUT][FROM_ARGV]: {args_text}", stdout_text)
                self.assertIn(f"[STDERR][FROM_ARGV]: {args_text}", stderr_text)
            else:
                self.assertIn(f"[STDOUT][FROM_STDIN]: {input_text}", stdout_text)
                self.assertIn(f"[STDERR][FROM_STDIN]: {input_text}", stderr_text)

    def stdout_redirection(self, console: str, with_env: bool = False):
        """Test only stdout redirection."""
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")

        argv_text = "output with\n multiline"
        # By default unix terminals the ONLCR flag is enabled. which replaces '\n' with '\r\n'
        # see https://man7.org/linux/man-pages/man3/termios.3.html.
        # This does not affect writing to normal files.
        argv_replaced_text = argv_text.replace("\n", "\r\n")

        program_args = [argv_text]
        env_text = "string from env"
        env = {"FROM_ENV": env_text} if with_env else {}

        with NamedTemporaryFile("rt") as stdout:
            self.launch_and_configurationDone(
                program,
                stdio=[None, stdout.name],
                console=console,
                args=program_args,
                env=env,
            )
            self.verify_process_exited()

            # check stdout
            stdout_text = stdout.read()
            stderr_text = self._get_debuggee_stderr()
            if with_env:
                self.assertIn(f"[STDOUT][FROM_ENV]: {env_text}", stdout_text)
                self.assertIn(f"[STDERR][FROM_ENV]: {env_text}", stderr_text)

                self.assertNotIn(f"[STDERR][FROM_ENV]: {env_text}", stdout_text)
                self.assertNotIn(f"[STDOUT][FROM_ENV]: {env_text}", stderr_text)
            else:
                self.assertIn(f"[STDOUT][FROM_ARGV]: {argv_text}", stdout_text)

                self.assertNotIn(
                    f"[STDERR][FROM_ARGV]: {argv_replaced_text}", stdout_text
                )
                self.assertNotIn(f"[STDOUT][FROM_ARGV]: {argv_text}", stderr_text)

            # check stderr
            stderr_text = self._get_debuggee_stderr()
            # FIXME: when using 'integrated' or 'external' terminal we do not correctly
            # escape newlines that are sent to the terminal.
            if console == "integratedConsole":
                if with_env:
                    self.assertNotIn(f"[STDOUT][FROM_ENV]: {env_text}", stderr_text)
                    self.assertIn(f"[STDERR][FROM_ENV]: {env_text}", stderr_text)
                else:
                    self.assertNotIn(
                        f"[STDOUT][FROM_ARGV]: {argv_replaced_text}", stderr_text
                    )
                    self.assertIn(
                        f"[STDERR][FROM_ARGV]: {argv_replaced_text}", stderr_text
                    )

    def stderr_redirection(self, console: str, with_env: bool = False):
        """Test only stdout redirection."""
        self.build_and_create_debug_adapter()
        program = self.getBuildArtifact("a.out")

        argv_text = "output with\n multiline"
        # By default unix terminals the ONLCR flag is enabled. which replaces '\n' with '\r\n'
        # see https://man7.org/linux/man-pages/man3/termios.3.html.
        # This does not affect writing to normal files.
        # Currently out test implementation for external and integrated Terminal does not run the
        # program through a shell terminal.
        argv_replaced_text = argv_text
        if console == "internalConsole":
            argv_replaced_text = argv_text.replace("\n", "\r\n")
        program_args = [argv_text]
        env_text = "string from env"
        env = {"FROM_ENV": env_text} if with_env else {}

        with NamedTemporaryFile("rt") as stderr:
            self.launch_and_configurationDone(
                program,
                stdio=[None, None, stderr.name],
                console=console,
                args=program_args,
                env=env,
            )
            self.verify_process_exited()
            stdout_text = self._get_debuggee_stdout()
            stderr_text = stderr.read()
            if with_env:
                self.assertIn(f"[STDOUT][FROM_ENV]: {env_text}", stdout_text)
                self.assertIn(f"[STDERR][FROM_ENV]: {env_text}", stderr_text)
            else:
                self.assertIn(f"[STDOUT][FROM_ARGV]: {argv_replaced_text}", stdout_text)
                self.assertIn(f"[STDERR][FROM_ARGV]: {argv_text}", stderr_text)

    @abstractmethod
    def _get_debuggee_stdout(self) -> str:
        """Retrieves the standard output (stdout) from the debuggee process.

        The default destination of the debuggee's stdout can vary based on how the debugger
        was launched (either a debug console or a pseudo-terminal (pty)).
        It requires subclasses to implement the specific mechanism for obtaining the stdout stream.
        """
        raise RuntimeError(f"NotImplemented for {self}")

    @abstractmethod
    def _get_debuggee_stderr(self) -> str:
        """Retrieves the standard error (stderr) from the debuggee process.

        The default destination of the debuggee's stderr can vary based on how the debugger
        was launched (either a debug console or a pseudo-terminal (pty)).
        It requires subclasses to implement the specific mechanism for obtaining the stderr stream.
        """
        raise RuntimeError(f"NotImplemented for {self}")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

from functionalities.breakpoint.hardware_breakpoints.base import *


class SimpleHWBreakpointTest(HardwareBreakpointTestBase):
    @skipTestIfFn(HardwareBreakpointTestBase.hw_breakpoints_unsupported)
    def test(self):
        """Test SBBreakpoint::SetIsHardware"""
        self.build()

        # Set a breakpoint on main.
        target, process, _, main_bp = lldbutil.run_to_source_breakpoint(
            self, "main", lldb.SBFileSpec("main.c")
        )

        break_on_me_bp = target.BreakpointCreateByLocation("main.c", 1)

        self.assertFalse(main_bp.IsHardware())
        self.assertFalse(break_on_me_bp.IsHardware())
        self.assertGreater(break_on_me_bp.GetNumResolvedLocations(), 0)

        error = break_on_me_bp.SetIsHardware(True)

        # Regardless of whether we succeeded in updating all the locations, the
        # breakpoint will be marked as a hardware breakpoint.
        self.assertTrue(break_on_me_bp.IsHardware())

        self.assertSuccess(error)

        # Continue to our Hardware breakpoint and verify that's the reason
        # we're stopped.
        process.Continue()
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )



"""
Check for an issue where capping does not work because the Target pointer appears to be changing behind our backs
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SyntheticCappingTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Check for an issue where capping does not work because the Target pointer appears to be changing behind our backs."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        process = self.dbg.GetSelectedTarget().GetProcess()

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)
            self.runCmd("settings set target.max-children-count 24", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # set up the synthetic children provider
        self.runCmd("script from fooSynthProvider import *")
        self.runCmd("type synth add -l fooSynthProvider foo")

        # note that the value of fake_a depends on target byte order
        if process.GetByteOrder() == lldb.eByteOrderLittle:
            fake_a_val = 0x02000000
        else:
            fake_a_val = 0x00000100

        # check that the synthetic children work, so we know we are doing the
        # right thing
        self.expect(
            "frame variable f00_1",
            substrs=[
                "a = 1",
                "fake_a = %d" % fake_a_val,
                "r = 34",
            ],
        )
        # num_children() should be called with at most max_num_children=25
        # (target.max-children-count + 1)
        self.expect(
            "script fooSynthProvider.reset_max_num_children_max()", substrs=["25"]
        )

        # check that capping works
        self.runCmd("settings set target.max-children-count 2", check=False)

        self.expect(
            "frame variable f00_1",
            substrs=[
                "a = 1",
                "fake_a = %d" % fake_a_val,
                "...",
            ],
        )
        self.expect(
            "script fooSynthProvider.reset_max_num_children_max()", substrs=["3"]
        )

        self.expect("frame variable f00_1", matching=False, substrs=["r = 34"])

        self.runCmd("settings set target.max-children-count 256", check=False)

        self.expect("frame variable f00_1", matching=True, substrs=["r = 34"])
        self.expect(
            "script fooSynthProvider.reset_max_num_children_max()", substrs=["257"]
        )



"""Test queues inspection SB APIs."""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestQueues(TestBase):
    @skipUnlessDarwin
    @add_test_categories(["pyapi"])
    def test_with_python_api_queues(self):
        """Test queues inspection SB APIs."""
        self.build()
        self.queues()

    @skipUnlessDarwin
    @add_test_categories(["pyapi"])
    def test_queue_specific_breakpoints(self):
        self.build()
        self.queue_specific_breakpoints()

    @skipUnlessDarwin
    @add_test_categories(["pyapi"])
    def test_with_python_api_queues_with_backtrace(self):
        """Test queues inspection SB APIs."""
        self.build()
        self.queues_with_libBacktraceRecording()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"

    def check_queue_for_valid_queue_id(self, queue):
        self.assertNotEqual(
            queue.GetQueueID(),
            0,
            "Check queue %s for valid QueueID (got 0x%x)"
            % (queue.GetName(), queue.GetQueueID()),
        )

    def check_running_and_pending_items_on_queue(
        self, queue, expected_running, expected_pending
    ):
        self.assertEqual(
            queue.GetNumPendingItems(),
            expected_pending,
            "queue %s should have %d pending items, instead has %d pending items"
            % (queue.GetName(), expected_pending, (queue.GetNumPendingItems())),
        )
        self.assertEqual(
            queue.GetNumRunningItems(),
            expected_running,
            "queue %s should have %d running items, instead has %d running items"
            % (queue.GetName(), expected_running, (queue.GetNumRunningItems())),
        )

    def describe_threads(self):
        desc = []
        for x in self.inferior_process:
            id = x.GetIndexID()
            reason_str = lldbutil.stop_reason_to_str(x.GetStopReason())

            location = "\t".join(
                [
                    lldbutil.get_description(x.GetFrameAtIndex(i))
                    for i in range(x.GetNumFrames())
                ]
            )
            desc.append(
                "thread %d: %s (queue id: %s) at\n\t%s"
                % (id, reason_str, x.GetQueueID(), location)
            )
        print("\n".join(desc))

    def check_number_of_threads_owned_by_queue(self, queue, number_threads):
        if queue.GetNumThreads() != number_threads:
            self.describe_threads()

        self.assertEqual(
            queue.GetNumThreads(),
            number_threads,
            "queue %s should have %d thread executing, but has %d"
            % (queue.GetName(), number_threads, queue.GetNumThreads()),
        )

    def check_queue_kind(self, queue, kind):
        expected_kind_string = "Unknown"
        if kind == lldb.eQueueKindSerial:
            expected_kind_string = "Serial queue"
        if kind == lldb.eQueueKindConcurrent:
            expected_kind_string = "Concurrent queue"
        actual_kind_string = "Unknown"
        if queue.GetKind() == lldb.eQueueKindSerial:
            actual_kind_string = "Serial queue"
        if queue.GetKind() == lldb.eQueueKindConcurrent:
            actual_kind_string = "Concurrent queue"
        self.assertEqual(
            queue.GetKind(),
            kind,
            "queue %s is expected to be a %s but it is actually a %s"
            % (queue.GetName(), expected_kind_string, actual_kind_string),
        )

    def check_queues_threads_match_queue(self, queue):
        for idx in range(0, queue.GetNumThreads()):
            t = queue.GetThreadAtIndex(idx)
            self.assertTrue(
                t.IsValid(),
                "Queue %s's thread #%d must be valid" % (queue.GetName(), idx),
            )
            self.assertEqual(
                t.GetQueueID(),
                queue.GetQueueID(),
                "Queue %s has a QueueID of %d but its thread #%d has a QueueID of %d"
                % (queue.GetName(), queue.GetQueueID(), idx, t.GetQueueID()),
            )
            self.assertEqual(
                t.GetQueueName(),
                queue.GetName(),
                "Queue %s has a QueueName of %s but its thread #%d has a QueueName of %s"
                % (queue.GetName(), queue.GetName(), idx, t.GetQueueName()),
            )
            self.assertEqual(
                t.GetQueue().GetQueueID(),
                queue.GetQueueID(),
                "Thread #%d's Queue's QueueID of %d is not the same as the QueueID of its owning queue %d"
                % (idx, t.GetQueue().GetQueueID(), queue.GetQueueID()),
            )

    def check_queue_breakpoints(self, queue1, queue2, queue_breakpoint):
        queue1_thread = queue1.GetThreadAtIndex(0)
        queue2_thread = queue2.GetThreadAtIndex(0)

        self.assertEqual(
            queue_breakpoint.GetQueueName(),
            queue1.GetName(),
            "The breakpoint was set for queue %s, but the breakpoint's queue name is %s"
            % (queue_breakpoint.GetQueueName(), queue1.GetName()),
        )
        self.assertEqual(
            queue_breakpoint.GetHitCount(),
            1,
            "The breakpoint for queue %s has not been hit"
            % (queue_breakpoint.GetQueueName()),
        )
        self.assertStopReason(
            queue1_thread.GetStopReason(),
            lldb.eStopReasonBreakpoint,
            "Queue %s is not stopped at breakpoint %d"
            % (queue1.GetName(), queue_breakpoint.GetID()),
        )
        self.assertNotEqual(
            queue2_thread.GetStopReason(),
            lldb.eStopReasonBreakpoint,
            "Queue %s is stopped at breakpoint %d, but this breakpoint should only be hit for queue %s"
            % (
                queue2.GetName(),
                queue_breakpoint.GetID(),
                queue_breakpoint.GetQueueName(),
            ),
        )

    def queues(self):
        """Test queues inspection SB APIs without libBacktraceRecording."""
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        break1 = target.BreakpointCreateByName("stopper", "a.out")
        self.assertTrue(break1, VALID_BREAKPOINT)
        process = target.LaunchSimple([], None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break1)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint 1.")

        self.inferior_process = process

        queue_submittor_1 = lldb.SBQueue()
        queue_performer_1 = lldb.SBQueue()
        queue_performer_2 = lldb.SBQueue()
        queue_performer_3 = lldb.SBQueue()
        for idx in range(0, process.GetNumQueues()):
            q = process.GetQueueAtIndex(idx)
            if q.GetName() == "com.apple.work_submittor_1":
                queue_submittor_1 = q
            if q.GetName() == "com.apple.work_performer_1":
                queue_performer_1 = q
            if q.GetName() == "com.apple.work_performer_2":
                queue_performer_2 = q
            if q.GetName() == "com.apple.work_performer_3":
                queue_performer_3 = q

        self.assertTrue(
            queue_submittor_1.IsValid()
            and queue_performer_1.IsValid()
            and queue_performer_2.IsValid()
            and queue_performer_3.IsValid(),
            "Got all four expected queues: %s %s %s %s"
            % (
                queue_submittor_1.IsValid(),
                queue_performer_1.IsValid(),
                queue_performer_2.IsValid(),
                queue_performer_3.IsValid(),
            ),
        )

        self.check_queue_for_valid_queue_id(queue_submittor_1)
        self.check_queue_for_valid_queue_id(queue_performer_1)
        self.check_queue_for_valid_queue_id(queue_performer_2)
        self.check_queue_for_valid_queue_id(queue_performer_3)

        self.check_number_of_threads_owned_by_queue(queue_submittor_1, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_1, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_2, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_3, 4)

        self.check_queue_kind(queue_submittor_1, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_1, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_2, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_3, lldb.eQueueKindConcurrent)

        self.check_queues_threads_match_queue(queue_submittor_1)
        self.check_queues_threads_match_queue(queue_performer_1)
        self.check_queues_threads_match_queue(queue_performer_2)
        self.check_queues_threads_match_queue(queue_performer_3)

        # We have threads running with all the different dispatch QoS service
        # levels - find those threads and check that we can get the correct
        # QoS name for each of them.

        user_initiated_thread = lldb.SBThread()
        user_interactive_thread = lldb.SBThread()
        utility_thread = lldb.SBThread()
        background_thread = lldb.SBThread()
        for th in process.threads:
            if th.GetName() == "user initiated QoS":
                user_initiated_thread = th
            if th.GetName() == "user interactive QoS":
                user_interactive_thread = th
            if th.GetName() == "utility QoS":
                utility_thread = th
            if th.GetName() == "background QoS":
                background_thread = th

        self.assertTrue(
            user_initiated_thread.IsValid(), "Found user initiated QoS thread"
        )
        self.assertTrue(
            user_interactive_thread.IsValid(), "Found user interactive QoS thread"
        )
        self.assertTrue(utility_thread.IsValid(), "Found utility QoS thread")
        self.assertTrue(background_thread.IsValid(), "Found background QoS thread")

        stream = lldb.SBStream()
        self.assertTrue(
            user_initiated_thread.GetInfoItemByPathAsString(
                "requested_qos.printable_name", stream
            ),
            "Get QoS printable string for user initiated QoS thread",
        )
        self.assertEqual(
            stream.GetData(),
            "User Initiated",
            "user initiated QoS thread name is valid",
        )
        stream.Clear()
        self.assertTrue(
            user_interactive_thread.GetInfoItemByPathAsString(
                "requested_qos.printable_name", stream
            ),
            "Get QoS printable string for user interactive QoS thread",
        )
        self.assertEqual(
            stream.GetData(),
            "User Interactive",
            "user interactive QoS thread name is valid",
        )
        stream.Clear()
        self.assertTrue(
            utility_thread.GetInfoItemByPathAsString(
                "requested_qos.printable_name", stream
            ),
            "Get QoS printable string for utility QoS thread",
        )
        self.assertEqual(
            stream.GetData(), "Utility", "utility QoS thread name is valid"
        )
        stream.Clear()
        self.assertTrue(
            background_thread.GetInfoItemByPathAsString(
                "requested_qos.printable_name", stream
            ),
            "Get QoS printable string for background QoS thread",
        )
        self.assertEqual(
            stream.GetData(), "Background", "background QoS thread name is valid"
        )

    @skipIfDarwin  # rdar://50379398
    def queues_with_libBacktraceRecording(self):
        """Test queues inspection SB APIs with libBacktraceRecording present."""
        exe = self.getBuildArtifact("a.out")

        if not os.path.isfile(
            "/Applications/Xcode.app/Contents/Developer/usr/lib/libBacktraceRecording.dylib"
        ):
            self.skipTest(
                "Skipped because libBacktraceRecording.dylib was present on the system."
            )

        if not os.path.isfile("/usr/lib/system/introspection/libdispatch.dylib"):
            self.skipTest(
                "Skipped because introspection libdispatch dylib is not present."
            )

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        break1 = target.BreakpointCreateByName("stopper", "a.out")
        self.assertTrue(break1, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        libbtr_path = "/Applications/Xcode.app/Contents/Developer/usr/lib/libBacktraceRecording.dylib"
        if self.getArchitecture() in [
            "arm",
            "arm64",
            "arm64e",
            "arm64_32",
            "armv7",
            "armv7k",
        ]:
            libbtr_path = "/Developer/usr/lib/libBacktraceRecording.dylib"

        process = target.LaunchSimple(
            [],
            [
                "DYLD_INSERT_LIBRARIES=%s" % (libbtr_path),
                "DYLD_LIBRARY_PATH=/usr/lib/system/introspection",
            ],
            self.get_process_working_directory(),
        )

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break1)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint 1.")

        self.inferior_process = process

        libbtr_module_filespec = lldb.SBFileSpec("libBacktraceRecording.dylib")
        libbtr_module = target.FindModule(libbtr_module_filespec)
        if not libbtr_module.IsValid():
            self.skipTest(
                "Skipped because libBacktraceRecording.dylib was not loaded into the process."
            )

        self.assertGreaterEqual(
            process.GetNumQueues(), 4, "Found the correct number of queues."
        )

        queue_submittor_1 = lldb.SBQueue()
        queue_performer_1 = lldb.SBQueue()
        queue_performer_2 = lldb.SBQueue()
        queue_performer_3 = lldb.SBQueue()
        for idx in range(0, process.GetNumQueues()):
            q = process.GetQueueAtIndex(idx)
            if "LLDB_COMMAND_TRACE" in os.environ:
                print("Queue  with id %s has name %s" % (q.GetQueueID(), q.GetName()))
            if q.GetName() == "com.apple.work_submittor_1":
                queue_submittor_1 = q
            if q.GetName() == "com.apple.work_performer_1":
                queue_performer_1 = q
            if q.GetName() == "com.apple.work_performer_2":
                queue_performer_2 = q
            if q.GetName() == "com.apple.work_performer_3":
                queue_performer_3 = q
            if q.GetName() == "com.apple.main-thread":
                if q.GetNumThreads() == 0:
                    print("Cannot get thread <=> queue associations")
                    return

        self.assertTrue(
            queue_submittor_1.IsValid()
            and queue_performer_1.IsValid()
            and queue_performer_2.IsValid()
            and queue_performer_3.IsValid(),
            "Got all four expected queues: %s %s %s %s"
            % (
                queue_submittor_1.IsValid(),
                queue_performer_1.IsValid(),
                queue_performer_2.IsValid(),
                queue_performer_3.IsValid(),
            ),
        )

        self.check_queue_for_valid_queue_id(queue_submittor_1)
        self.check_queue_for_valid_queue_id(queue_performer_1)
        self.check_queue_for_valid_queue_id(queue_performer_2)
        self.check_queue_for_valid_queue_id(queue_performer_3)

        self.check_running_and_pending_items_on_queue(queue_submittor_1, 1, 0)
        self.check_running_and_pending_items_on_queue(queue_performer_1, 1, 3)
        self.check_running_and_pending_items_on_queue(queue_performer_2, 1, 9999)
        self.check_running_and_pending_items_on_queue(queue_performer_3, 4, 0)

        self.check_number_of_threads_owned_by_queue(queue_submittor_1, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_1, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_2, 1)
        self.check_number_of_threads_owned_by_queue(queue_performer_3, 4)

        self.check_queue_kind(queue_submittor_1, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_1, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_2, lldb.eQueueKindSerial)
        self.check_queue_kind(queue_performer_3, lldb.eQueueKindConcurrent)

        self.check_queues_threads_match_queue(queue_submittor_1)
        self.check_queues_threads_match_queue(queue_performer_1)
        self.check_queues_threads_match_queue(queue_performer_2)
        self.check_queues_threads_match_queue(queue_performer_3)

        self.assertTrue(
            queue_performer_2.GetPendingItemAtIndex(0).IsValid(),
            "queue 2's pending item #0 is valid",
        )
        self.assertEqual(
            queue_performer_2.GetPendingItemAtIndex(0)
            .GetAddress()
            .GetSymbol()
            .GetName(),
            "doing_the_work_2",
            "queue 2's pending item #0 should be doing_the_work_2",
        )
        self.assertEqual(
            queue_performer_2.GetNumPendingItems(),
            9999,
            "verify that queue 2 still has 9999 pending items",
        )
        self.assertTrue(
            queue_performer_2.GetPendingItemAtIndex(9998).IsValid(),
            "queue 2's pending item #9998 is valid",
        )
        self.assertEqual(
            queue_performer_2.GetPendingItemAtIndex(9998)
            .GetAddress()
            .GetSymbol()
            .GetName(),
            "doing_the_work_2",
            "queue 2's pending item #0 should be doing_the_work_2",
        )
        self.assertFalse(
            queue_performer_2.GetPendingItemAtIndex(9999).IsValid(),
            "queue 2's pending item #9999 is invalid",
        )

    def queue_specific_breakpoints(self):
        # Run the executable until the stopper function and get the breakpoint
        # that's created from that. Then set the queue name of the breakpoint
        # to be the name of the main thread
        (
            target,
            process,
            main_thread,
            queue_breakpoint,
        ) = lldbutil.run_to_name_breakpoint(self, "stopper", only_one_thread=False)
        queue_breakpoint.SetQueueName(main_thread.GetQueue().GetName())

        # Create a submittor queue
        queue_submittor_1 = lldb.SBQueue()
        for idx in range(0, process.GetNumQueues()):
            q = process.GetQueueAtIndex(idx)
            if q.GetName() == "com.apple.work_submittor_1":
                queue_submittor_1 = q

        self.assertTrue(
            queue_submittor_1.IsValid(),
            "Unable to get expected queue com.apple.work_submittor_1, instead got queue %s"
            % (queue_submittor_1.GetName()),
        )

        self.check_queue_breakpoints(
            main_thread.GetQueue(), queue_submittor_1, queue_breakpoint
        )



import os
import os.path
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.gdbclientutils import *


class GDBRemoteTestBase(TestBase):
    """
    Base class for GDB client tests.

    This class will setup and start a mock GDB server for the test to use.
    It also provides assertPacketLogReceived, which simplifies the checking
    of packets sent by the client.
    """

    NO_DEBUG_INFO_TESTCASE = True
    server = None
    server_socket_class = TCPServerSocket

    def setUp(self):
        TestBase.setUp(self)
        self.server = MockGDBServer(self.server_socket_class())
        self.server.start()

    def tearDown(self):
        # TestBase.tearDown will kill the process, but we need to kill it early
        # so its client connection closes and we can stop the server before
        # finally calling the base tearDown.
        if self.process() is not None:
            self.process().Kill()
        self.server.stop()
        TestBase.tearDown(self)

    def createTarget(self, yaml_path):
        """
        Create a target by auto-generating the object based on the given yaml
        instructions.

        This will track the generated object so it can be automatically removed
        during tearDown.
        """
        yaml_base, ext = os.path.splitext(yaml_path)
        obj_path = self.getBuildArtifact(yaml_base)
        self.yaml2obj(yaml_path, obj_path)
        return self.dbg.CreateTarget(obj_path)

    def connect(self, target, plugin="gdb-remote"):
        """
        Create a process by connecting to the mock GDB server.

        Includes assertions that the process was successfully created.
        """
        listener = self.dbg.GetListener()
        error = lldb.SBError()
        process = target.ConnectRemote(
            listener, self.server.get_connect_url(), plugin, error
        )
        self.assertTrue(error.Success(), error.description)
        self.assertTrue(process, PROCESS_IS_VALID)
        return process

    def assertPacketLogReceived(self, packets, log: PacketLog = None):
        """
        Assert that the mock server's packet log received the given packets.

        The packet log includes all packets sent by the client and received
        by the server.  This function makes it easy to verify that the client
        sent the expected packets to the server.

        The check does not require that the packets be consecutive, but does
        require that they are ordered in the log as they ordered in the arg.
        """
        if log is None:
            received = self.server.responder.packetLog.get_received()
        else:
            received = log.get_received()
        i = 0
        j = 0

        while i < len(packets) and j < len(received):
            if received[j] == packets[i]:
                i += 1
            j += 1
        if i < len(packets):
            self.fail(
                "Did not receive: %s\nLast 10 packets:\n\t%s"
                % (packets[i], "\n\t".join(received))
            )


class GDBPlatformClientTestBase(GDBRemoteTestBase):
    """
    Base class for platform server clients.

    This class extends GDBRemoteTestBase by automatically connecting
    via "platform connect" in the setUp() method.
    """

    def setUp(self):
        super().setUp()
        self.runCmd("platform select remote-gdb-server")
        self.runCmd("platform connect " + self.server.get_connect_url())
        self.assertTrue(self.dbg.GetSelectedPlatform().IsConnected())

    def tearDown(self):
        self.dbg.GetSelectedPlatform().DisconnectRemote()
        super().tearDown()



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test_summary_string_with_bare_dollar_char(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))
        self.runCmd("type summary add --summary-string '$ $CASH $' --no-value Dollars")
        self.expect("v cash", startstr="(Dollars) cash = $ $CASH $")

    def test_summary_string_with_bare_dollar_char_before_var(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))
        self.runCmd("type summary add --summary-string '$${var}' --no-value Dollars")
        self.expect("v cash", startstr="(Dollars) cash = $99")



"""
Test that variable expressions of integer basic types are evaluated correctly.
"""

import AbstractBase

from lldbsuite.test.decorators import *


class LongTypesExprTestCase(AbstractBase.GenericTester):
    def test_long_type(self):
        """Test that long-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("long.cpp", ["long"])

    @skipUnlessDarwin
    def test_long_type_from_block(self):
        """Test that long-type variables are displayed correctly from a block."""
        self.build_and_run_expr("long.cpp", ["long"], bc=True)

    def test_unsigned_long_type(self):
        """Test that 'unsigned long'-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("unsigned_long.cpp", ["unsigned", "long"])

    @skipUnlessDarwin
    def test_unsigned_long_type_from_block(self):
        """Test that 'unsigned_long'-type variables are displayed correctly from a block."""
        self.build_and_run_expr("unsigned_long.cpp", ["unsigned", "long"], bc=True)

    def test_long_long_type(self):
        """Test that 'long long'-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("long_long.cpp", ["long long"])

    @skipUnlessDarwin
    def test_long_long_type_from_block(self):
        """Test that 'long_long'-type variables are displayed correctly from a block."""
        self.build_and_run_expr("long_long.cpp", ["long long"], bc=True)

    def test_unsigned_long_long_type(self):
        """Test that 'unsigned long long'-type variable expressions are evaluated correctly."""
        self.build_and_run_expr("unsigned_long_long.cpp", ["unsigned", "long long"])

    @skipUnlessDarwin
    def test_unsigned_long_long_type_from_block(self):
        """Test that 'unsigned_long_long'-type variables are displayed correctly from a block."""
        self.build_and_run_expr(
            "unsigned_long_long.cpp", ["unsigned", "long long"], bc=True
        )



import os
import itertools
import platform
import re
import subprocess
import sys

import lit.util
from lit.formats import ShTest
from lit.llvm import llvm_config
from lit.llvm.subst import FindTool
from lit.llvm.subst import ToolSubst

import posixpath

def _get_lldb_init_path(config):
    return os.path.join(config.test_exec_root, "lit-lldb-init-quiet")


def _disallow(config, execName):
    warning = """
    echo '*** Do not use \'{0}\' in tests; use \'%''{0}\'. ***' &&
    exit 1 && echo
  """
    config.substitutions.append((" {0} ".format(execName), warning.format(execName)))


def get_lldb_args(config, suffix=""):
    lldb_args = []
    if "remote-linux" in config.available_features:
        lldb_args += [
            "-O",
            '"platform select remote-linux"',
            "-O",
            f'"platform connect {config.lldb_platform_url}"',
        ]
        if config.lldb_platform_working_dir:
            dir = posixpath.join(f"{config.lldb_platform_working_dir}", "shell")
            if suffix:
                dir += posixpath.join(dir, f"{suffix}")
            lldb_args += [
                "-O",
                f'"platform shell mkdir -p {dir}"',
                "-O",
                f'"platform settings -w {dir}"',
            ]
    lldb_args += ["--no-lldbinit", "-S", _get_lldb_init_path(config)]
    return lldb_args


class ShTestLldb(ShTest):
    def __init__(
        self, execute_external=False, extra_substitutions=[], preamble_commands=[]
    ):
        super().__init__(execute_external, extra_substitutions, preamble_commands)

    def execute(self, test, litConfig):
        # Run each Shell test in a separate directory (on remote).

        # Find directory change command in %lldb substitution.
        for i, t in enumerate(test.config.substitutions):
            if re.match(t[0], "%lldb"):
                cmd = t[1]
                if '-O "platform settings -w ' in cmd:
                    # If command is present, it is added by get_lldb_args.
                    # Replace the path with the tests' path in suite.
                    # Example:
                    # bin/lldb
                    #   -O "platform shell mkdir -p /home/user/shell"
                    #   -O "platform settings -w /home/user/shell" ...
                    # =>
                    # bin/lldb
                    #   -O "platform shell mkdir -p /home/user/shell/SymbolFile/Breakpad/inline-record.test"
                    #   -O "platform settings -w /home/user/shell/SymbolFile/Breakpad/inline-record.test" ...
                    args_def = " ".join(get_lldb_args(test.config))
                    args_unique = " ".join(
                        get_lldb_args(
                            test.config,
                            posixpath.join(*test.path_in_suite),
                        )
                    )
                    test.config.substitutions[i] = (
                        t[0],
                        cmd.replace(args_def, args_unique),
                    )
                break
        return super().execute(test, litConfig)


def use_lldb_substitutions(config):
    # Set up substitutions for primary tools.  These tools must come from config.lldb_tools_dir
    # which is basically the build output directory.  We do not want to find these in path or
    # anywhere else, since they are specifically the programs which are actually being tested.

    dsname = "debugserver" if platform.system() in ["Darwin"] else "lldb-server"
    dsargs = [] if platform.system() in ["Darwin"] else ["gdbserver"]

    build_script = os.path.dirname(__file__)
    build_script = os.path.join(build_script, "build.py")
    build_script_args = [
        build_script,
        (
            "--compiler=clang" if config.enable_remote else "--compiler=any"
        ),  # Default to best compiler
        "--arch=" + str(config.lldb_bitness),
    ]
    if config.lldb_lit_tools_dir:
        build_script_args.append("--tools-dir={0}".format(config.lldb_lit_tools_dir))
    if config.lldb_tools_dir:
        build_script_args.append("--tools-dir={0}".format(config.lldb_tools_dir))
    if config.llvm_libs_dir:
        build_script_args.append("--libs-dir={0}".format(config.llvm_libs_dir))
    if config.objc_gnustep_dir:
        build_script_args.append(
            '--objc-gnustep-dir="{0}"'.format(config.objc_gnustep_dir)
        )
    if config.cmake_sysroot:
        build_script_args.append("--sysroot={0}".format(config.cmake_sysroot))

    lldb_init = _get_lldb_init_path(config)
    launcher = getattr(config, "lldb_launcher", None)

    primary_tools = [
        ToolSubst(
            "%lldb",
            command=FindTool("lldb"),
            extra_args=get_lldb_args(config),
            launcher=launcher,
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-init",
            command=FindTool("lldb"),
            extra_args=["-S", lldb_init],
            launcher=launcher,
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-noinit",
            command=FindTool("lldb"),
            extra_args=["--no-lldbinit"],
            launcher=launcher,
            unresolved="fatal",
        ),
        ToolSubst(
            "%lldb-server",
            command=FindTool("lldb-server"),
            extra_args=[],
            unresolved="ignore",
        ),
        ToolSubst(
            "%debugserver",
            command=FindTool(dsname),
            extra_args=dsargs,
            unresolved="ignore",
        ),
        ToolSubst(
            "%platformserver",
            command=FindTool("lldb-server"),
            extra_args=["platform"],
            unresolved="ignore",
        ),
        ToolSubst(
            "%lldb-rpc-gen",
            command=FindTool("lldb-rpc-gen"),
            # We need the LLDB build directory root to pass into the tool, not the test build root.
            extra_args=[
                "-p " + config.lldb_build_directory + "/..",
                '--extra-arg="-resource-dir=' + config.clang_resource_dir + '"',
            ],
            unresolved="ignore",
        ),
        "lldb-test",
        "lldb-dap",
        ToolSubst(
            "%build", command="'" + sys.executable + "'", extra_args=build_script_args
        ),
    ]

    _disallow(config, "lldb")
    _disallow(config, "lldb-server")
    _disallow(config, "debugserver")
    _disallow(config, "platformserver")

    llvm_config.add_tool_substitutions(primary_tools, [config.lldb_tools_dir])


def _use_msvc_substitutions(config):
    # If running from a Visual Studio Command prompt (e.g. vcvars), this will
    # detect the include and lib paths, and find cl.exe and link.exe and create
    # substitutions for each of them that explicitly specify /I and /L paths
    cl = lit.util.which("cl")

    if not cl:
        return

    # Don't use lit.util.which() for link.exe: In `git bash`, it will pick
    # up /usr/bin/link (another name for ln).
    link = os.path.join(os.path.dirname(cl), "link.exe")

    cl = '"' + cl + '"'
    link = '"' + link + '"'
    includes = os.getenv("INCLUDE", "").split(";")
    libs = os.getenv("LIB", "").split(";")

    config.available_features.add("msvc")
    compiler_flags = ['"/I{}"'.format(x) for x in includes if os.path.exists(x)]
    linker_flags = ['"/LIBPATH:{}"'.format(x) for x in libs if os.path.exists(x)]

    tools = [
        ToolSubst("%msvc_cl", command=cl, extra_args=compiler_flags),
        ToolSubst("%msvc_link", command=link, extra_args=linker_flags),
    ]
    llvm_config.add_tool_substitutions(tools)
    return


def use_support_substitutions(config):
    # Set up substitutions for support tools.  These tools can be overridden at the CMake
    # level (by specifying -DLLDB_LIT_TOOLS_DIR), installed, or as a last resort, we can use
    # the just-built version.
    if config.enable_remote:
        host_flags = ["--target=" + config.target_triple]
    else:
        host_flags = ["--target=" + config.host_triple]
    if platform.system() in ["Darwin"]:
        try:
            out = subprocess.check_output(["xcrun", "--show-sdk-path"]).strip()
            res = 0
        except OSError:
            res = -1
        if res == 0 and out:
            sdk_path = out.decode("utf-8")
            llvm_config.lit_config.note("using SDKROOT: %r" % sdk_path)
            host_flags += ["-isysroot", sdk_path]
    elif sys.platform != "win32":
        host_flags += ["-pthread"]

    if sys.platform.startswith("netbsd"):
        # needed e.g. to use freshly built libc++
        host_flags += [
            "-L" + config.llvm_libs_dir,
            "-Wl,-rpath," + config.llvm_libs_dir,
        ]

    # The clang module cache is used for building inferiors.
    host_flags += ["-fmodules-cache-path={}".format(config.clang_module_cache)]

    # Our files use x86 AT&T assembly throughout.
    # Enable it explicitly so any local Clang preference for Intel syntax gets overriden.
    if "x86-registered-target" in config.available_features:
        host_flags += ["-mllvm", "-x86-asm-syntax=att"]

    if config.cmake_sysroot:
        host_flags += ["--sysroot={}".format(config.cmake_sysroot)]

    if config.enable_remote and config.has_libcxx:
        host_flags += [
            "-L{}".format(config.libcxx_libs_dir),
            "-lc++",
        ]
    # By default, macOS doesn't allow injecting the ASAN runtime into system processes.
    if platform.system() in ["Darwin"] and config.llvm_use_sanitizer:
        system_clang = (
            subprocess.check_output(["xcrun", "-find", "clang"]).strip().decode("utf-8")
        )
        system_liblto = os.path.join(
            os.path.dirname(os.path.dirname(system_clang)), "lib", "libLTO.dylib"
        )
        host_flags += ["-Wl,-lto_library", "-Wl," + system_liblto]

    host_flags = " ".join(host_flags)
    config.substitutions.append(("%clang_host", "%clang " + host_flags))
    config.substitutions.append(("%clangxx_host", "%clangxx " + host_flags))
    config.substitutions.append(
        ("%clang_cl_host", "%clang_cl --target=" + config.host_triple)
    )

    additional_tool_dirs = []
    if config.lldb_lit_tools_dir:
        additional_tool_dirs.append(config.lldb_lit_tools_dir)

    llvm_config.use_clang(
        additional_flags=["--target=specify-a-target-or-use-a-_host-substitution"],
        additional_tool_dirs=additional_tool_dirs,
        required=True,
        use_installed=True,
    )
    if llvm_config.clang_has_bounds_safety():
        llvm_config.lit_config.note("clang has -fbounds-safety support")
        config.available_features.add("clang-bounds-safety")

    if sys.platform == "win32":
        _use_msvc_substitutions(config)

    have_lld = llvm_config.use_lld(
        additional_tool_dirs=additional_tool_dirs, required=False, use_installed=True
    )
    if have_lld:
        config.available_features.add("lld")

    support_tools = [
        "yaml2obj",
        "obj2yaml",
        "llvm-dwp",
        "llvm-pdbutil",
        "llvm-mc",
        "llvm-readobj",
        "llvm-objdump",
        "llvm-objcopy",
        "lli",
    ]
    additional_tool_dirs += [config.lldb_tools_dir, config.llvm_tools_dir]
    llvm_config.add_tool_substitutions(support_tools, additional_tool_dirs)

    _disallow(config, "clang")
